@@ -1,6 +1,6 @@
 SELECT MIN(mi.info) AS release_date,
        MIN(t.title) AS modern_american_internet_movie
-FROM aka_title AS at,
+FROM aka_title AS at_,
      company_name AS cn,
      company_type AS ct,
      info_type AS it1,
@@ -16,16 +16,16 @@ WHERE cn.country_code = '[us]'
   AND (mi.info LIKE 'USA:% 199%'
        OR mi.info LIKE 'USA:% 200%')
   AND t.production_year > 1990
-  AND t.id = at.movie_id
+  AND t.id = at_.movie_id
   AND t.id = mi.movie_id
   AND t.id = mk.movie_id
   AND t.id = mc.movie_id
   AND mk.movie_id = mi.movie_id
   AND mk.movie_id = mc.movie_id
-  AND mk.movie_id = at.movie_id
+  AND mk.movie_id = at_.movie_id
   AND mi.movie_id = mc.movie_id
-  AND mi.movie_id = at.movie_id
-  AND mc.movie_id = at.movie_id
+  AND mi.movie_id = at_.movie_id
+  AND mc.movie_id = at_.movie_id
   AND k.id = mk.keyword_id
   AND it1.id = mi.info_type_id
   AND cn.id = mc.company_id
