@@ -1,10 +1,9 @@
 #ifndef JEMALLOC_INTERNAL_BIN_TYPES_H
 #define JEMALLOC_INTERNAL_BIN_TYPES_H
 
+#include "jemalloc/internal/jemalloc_preamble.h"
 #include "jemalloc/internal/sc.h"
 
-namespace duckdb_jemalloc {
-
 #define BIN_SHARDS_MAX (1 << EDATA_BITS_BINSHARD_WIDTH)
 #define N_BIN_SHARDS_DEFAULT 1
 
@@ -16,6 +15,4 @@ struct tsd_binshards_s {
 	uint8_t binshard[SC_NBINS];
 };
 
-} // namespace duckdb_jemalloc
-
 #endif /* JEMALLOC_INTERNAL_BIN_TYPES_H */
