@@ -4,7 +4,7 @@ namespace duckdb {
 
 static void ValidFunction(DataChunk &args, ExpressionState &state, Vector &result) {
 	auto &lstate = JSONFunctionLocalState::ResetAndGet(state);
-	auto alc = lstate.json_allocator.GetYYAlc();
+	auto alc = lstate.json_allocator->GetYYAlc();
 	auto &inputs = args.data[0];
 	UnaryExecutor::Execute<string_t, bool>(inputs, result, args.size(), [&](string_t input) {
 		return JSONCommon::ReadDocumentUnsafe(input, JSONCommon::READ_FLAG, alc);
