@@ -21,6 +21,7 @@ from .statement import (
     Sleep,
     SleepUnit,
     Skip,
+    Unzip,
     SortStyle,
     Unskip,
 )
@@ -62,13 +63,21 @@ class ExecuteResult:
 
 BUILTIN_EXTENSIONS = [
     'json',
-    'fts',
-    'tpcds',
-    'tpch',
     'parquet',
     'icu',
 ]
 
+from duckdb import DuckDBPyConnection
+
+# def patch_execute(method):
+#    def patched_execute(self, *args, **kwargs):
+#        print(*args)
+#        return method(self, *args, **kwargs)
+#    return patched_execute
+
+# patched_execute = patch_execute(getattr(DuckDBPyConnection, "execute"))
+# setattr(DuckDBPyConnection, "execute", patched_execute)
+
 
 class SQLLogicStatementData:
     # Context information about a statement
@@ -83,10 +92,11 @@ class SQLLogicStatementData:
 
 
 class TestException(Exception):
+    __test__ = False
     __slots__ = ['data', 'message', 'result']
 
     def __init__(self, data: SQLLogicStatementData, message: str, result: ExecuteResult):
-        self.message = f'{str(data)} {message}'
+        self.message = message
         super().__init__(self.message)
         self.data = data
         self.result = result
@@ -123,6 +133,7 @@ class QueryResult:
     def row_count(self) -> int:
         return self._row_count
 
+    @property
     def column_count(self) -> int:
         assert self._column_count != 0
         return self._column_count
@@ -152,7 +163,7 @@ class QueryResult:
             context.fail(self.get_error())
 
         row_count = self.row_count()
-        column_count = self.column_count()
+        column_count = self.column_count
         total_value_count = row_count * column_count
 
         if len(values) == 1 and result_is_hash(values[0]):
@@ -168,7 +179,7 @@ class QueryResult:
             logger.output_result(self, result_values_string)
 
         if sort_style == SortStyle.ROW_SORT:
-            ncols = self.column_count()
+            ncols = self.column_count
             nrows = int(total_value_count / ncols)
             rows = [result_values_string[i * ncols : (i + 1) * ncols] for i in range(nrows)]
 
@@ -222,8 +233,8 @@ class QueryResult:
             original_expected_columns = expected_column_count
             column_count_mismatch = False
 
-            if expected_column_count != self.column_count():
-                expected_column_count = self.column_count()
+            if expected_column_count != self.column_count:
+                expected_column_count = self.column_count
                 column_count_mismatch = True
 
             expected_rows = len(comparison_values) / expected_column_count
@@ -238,7 +249,7 @@ class QueryResult:
                 row_wise = True
             elif len(comparison_values) % expected_column_count != 0:
                 if column_count_mismatch:
-                    logger.column_count_mismatch(self, query.values, original_expected_columns, row_wise)
+                    logger.column_count_mismatch(self, values, original_expected_columns, row_wise)
                 else:
                     logger.not_cleanly_divisible(expected_column_count, len(comparison_values))
                 # FIXME: the logger should just create the strings to send to self.fail()/self.skip()
@@ -246,7 +257,7 @@ class QueryResult:
 
             if expected_rows != self.row_count():
                 if column_count_mismatch:
-                    logger.column_count_mismatch(self, query.values, original_expected_columns, row_wise)
+                    logger.column_count_mismatch(self, values, original_expected_columns, row_wise)
                 else:
                     logger.wrong_row_count(
                         expected_rows, result_values_string, comparison_values, expected_column_count, row_wise
@@ -259,7 +270,7 @@ class QueryResult:
                     splits = [x for x in val.split("\t") if x != '']
                     if len(splits) != expected_column_count:
                         if column_count_mismatch:
-                            logger.column_count_mismatch(self, query.values, original_expected_columns, row_wise)
+                            logger.column_count_mismatch(self, values, original_expected_columns, row_wise)
                         logger.split_mismatch(i + 1, expected_column_count, len(splits))
                         context.fail("")
                     for c, split_val in enumerate(splits):
@@ -320,9 +331,9 @@ class QueryResult:
                 hash_compare_error = values[0] != hash_value
 
             if hash_compare_error:
-                expected_result = self.result_label_map.get(query_label)
-                logger.wrong_result_hash(expected_result, self)
-                self.fail_query(query)
+                expected_result = runner.result_label_map.get(query_label)
+                # logger.wrong_result_hash(expected_result, self)
+                context.fail(query)
 
             assert not hash_compare_error
 
@@ -393,6 +404,8 @@ class SQLLogicDatabase:
 
         # Now re-open the current database
         read_only = 'access_mode' in self.config and self.config['access_mode'] == 'read_only'
+        if 'access_mode' not in self.config:
+            self.config['access_mode'] = 'automatic'
         self.database = duckdb.connect(path, read_only, self.config)
 
         # Load any previously loaded extensions again
@@ -421,24 +434,50 @@ class SQLLogicDatabase:
         return SQLLogicConnectionPool(self.database.cursor())
 
 
+def is_regex(input: str) -> bool:
+    return input.startswith("<REGEX>:") or input.startswith("<!REGEX>:")
+
+
+def matches_regex(input: str, actual_str: str) -> bool:
+    if input.startswith("<REGEX>:"):
+        should_match = True
+        regex_str = input.replace("<REGEX>:", "")
+    else:
+        should_match = False
+        regex_str = input.replace("<!REGEX>:", "")
+    # The exact match will never be the same, allow leading and trailing messages
+    if regex_str[:2] != '.*':
+        regex_str = ".*" + regex_str
+    if regex_str[-2:] != '.*':
+        regex_str = regex_str + '.*'
+
+    re_options = re.DOTALL
+    re_pattern = re.compile(regex_str, re_options)
+    regex_matches = bool(re_pattern.fullmatch(actual_str))
+    return regex_matches == should_match
+
+
+def has_external_access(conn):
+    # this is required for the python tester to work, as we make use of replacement scans
+    try:
+        res = conn.sql("select current_setting('enable_external_access')").fetchone()[0]
+        return res
+    except duckdb.TransactionException:
+        return True
+    except duckdb.BinderException:
+        return True
+    except duckdb.InvalidInputException:
+        return True
+
+
 def compare_values(result: QueryResult, actual_str, expected_str, current_column):
     error = False
 
     if actual_str == expected_str:
         return True
 
-    if expected_str.startswith("<REGEX>:") or expected_str.startswith("<!REGEX>:"):
-        if expected_str.startswith("<REGEX>:"):
-            should_match = True
-            regex_str = expected_str.replace("<REGEX>:", "")
-        else:
-            should_match = False
-            regex_str = expected_str.replace("<!REGEX>:", "")
-        re_options = re.DOTALL
-        re_pattern = re.compile(regex_str, re_options)
-        regex_matches = bool(re_pattern.fullmatch(actual_str))
-        if regex_matches == should_match:
-            return True
+    if is_regex(expected_str):
+        return matches_regex(expected_str, actual_str)
 
     sql_type = result.types[current_column]
 
@@ -510,7 +549,7 @@ def result_is_file(result: str):
 def load_result_from_file(fname, result: QueryResult):
     con = duckdb.connect()
     con.execute(f"PRAGMA threads={os.cpu_count()}")
-    column_count = result.column_count()
+    column_count = result.column_count
 
     fname = fname.replace("<FILE>:", "")
 
@@ -568,7 +607,7 @@ def sql_logic_test_convert_value(value, sql_type, is_sqlite_test: bool) -> str:
 def duck_db_convert_result(result: QueryResult, is_sqlite_test: bool) -> List[str]:
     out_result = []
     row_count = result.row_count()
-    column_count = result.column_count()
+    column_count = result.column_count
 
     for r in range(row_count):
         for c in range(column_count):
@@ -680,7 +719,7 @@ class SQLLogicContext:
         # Apply a replacement for every registered keyword
         if '__BUILD_DIRECTORY__' in input:
             self.skiptest("Test contains __BUILD_DIRECTORY__ which isnt supported")
-        for key, value in self.keywords.items():
+        for key, value in self.keywords.items().__reversed__():
             input = input.replace(key, value)
         return input
 
@@ -724,6 +763,7 @@ class SQLLogicContext:
             Restart: self.execute_restart,
             HashThreshold: self.execute_hash_threshold,
             Set: self.execute_set,
+            Unzip: self.execute_unzip,
             Loop: self.execute_loop,
             Foreach: self.execute_foreach,
             Endloop: None,  # <-- should never be encountered outside of Loop/Foreach
@@ -756,7 +796,9 @@ class SQLLogicContext:
 
     def execute_load(self, load: Load):
         if self.in_loop():
-            self.fail("load cannot be called in a loop")
+            # FIXME: should add support for this, the CPP tester supports this
+            self.skiptest("load cannot be called in a loop")
+            # self.fail("load cannot be called in a loop")
 
         readonly = load.readonly
 
@@ -768,16 +810,19 @@ class SQLLogicContext:
                 self.runner.delete_database(dbpath)
         else:
             dbpath = ""
+        self.runner.loaded_databases.add(dbpath)
 
         # set up the config file
         additional_config = {}
         if readonly:
-            additional_config['temp_directory'] = False
+            additional_config['temp_directory'] = ""
             additional_config['access_mode'] = 'read_only'
         else:
-            additional_config['temp_directory'] = True
             additional_config['access_mode'] = 'automatic'
 
+        if load.version:
+            additional_config['storage_compatibility_version'] = str(load.version)
+
         self.pool = None
         self.runner.database = None
         self.runner.database = SQLLogicDatabase(dbpath, self, additional_config)
@@ -786,6 +831,8 @@ class SQLLogicContext:
     def execute_query(self, query: Query):
         assert isinstance(query, Query)
         conn = self.get_connection(query.connection_name)
+        if not has_external_access(conn):
+            self.skiptest("enable_external_access is explicitly disabled by the test")
         sql_query = '\n'.join(query.lines)
         sql_query = self.replace_keywords(sql_query)
 
@@ -809,27 +856,34 @@ class SQLLogicContext:
                     if 'returning' not in sql_query.lower():
                         return False
                     return True
+                if statement.type in [duckdb.StatementType.COPY]:
+                    if 'return_files' not in sql_query.lower():
+                        return False
+                    return True
                 return len(statement.expected_result_type) == 1
 
             if is_query_result(sql_query, statement):
                 original_rel = conn.query(sql_query)
-                original_types = original_rel.types
-                # We create new names for the columns, because they might be duplicated
-                aliased_columns = [f'c{i}' for i in range(len(original_types))]
+                if original_rel is None:
+                    query_result = QueryResult([(0,)], ['BIGINT'])
+                else:
+                    original_types = original_rel.types
+                    # We create new names for the columns, because they might be duplicated
+                    aliased_columns = [f'c{i}' for i in range(len(original_types))]
 
-                expressions = [f'"{name}"::VARCHAR' for name, sql_type in zip(aliased_columns, original_types)]
-                aliased_table = ", ".join(aliased_columns)
-                expression_list = ", ".join(expressions)
-                try:
-                    # Select from the result, converting the Values to the right type for comparison
-                    transformed_query = (
-                        f"select {expression_list} from original_rel unnamed_subquery_blabla({aliased_table})"
-                    )
-                    stringified_rel = conn.query(transformed_query)
-                except duckdb.Error as e:
-                    self.fail(f"Could not select from the ValueRelation: {str(e)}")
-                result = stringified_rel.fetchall()
-                query_result = QueryResult(result, original_types)
+                    expressions = [f'"{name}"::VARCHAR' for name, sql_type in zip(aliased_columns, original_types)]
+                    aliased_table = ", ".join(aliased_columns)
+                    expression_list = ", ".join(expressions)
+                    try:
+                        # Select from the result, converting the Values to the right type for comparison
+                        transformed_query = (
+                            f"select {expression_list} from original_rel unnamed_subquery_blabla({aliased_table})"
+                        )
+                        stringified_rel = conn.query(transformed_query)
+                    except duckdb.Error as e:
+                        self.fail(f"Could not select from the ValueRelation: {str(e)}")
+                    result = stringified_rel.fetchall()
+                    query_result = QueryResult(result, original_types)
             elif duckdb.ExpectedResultType.CHANGED_ROWS in statement.expected_result_type:
                 conn.execute(sql_query)
                 result = conn.fetchall()
@@ -849,6 +903,18 @@ class SQLLogicContext:
     def execute_skip(self, statement: Skip):
         self.runner.skip()
 
+    def execute_unzip(self, statement: Unzip):
+        import gzip
+        import shutil
+
+        source = self.replace_keywords(statement.source)
+        destination = self.replace_keywords(statement.destination)
+
+        with gzip.open(source, 'rb') as f_in:
+            with open(destination, 'wb') as f_out:
+                shutil.copyfileobj(f_in, f_out)
+        print(f"Extracted to '{destination}'")
+
     def execute_unskip(self, statement: Unskip):
         self.runner.unskip()
 
@@ -868,6 +934,7 @@ class SQLLogicContext:
         self.runner.database = SQLLogicDatabase(path, self)
         self.pool = self.runner.database.connect()
         con = self.pool.get_connection()
+
         for setting in old_settings:
             name, value = setting
             if name in [
@@ -877,23 +944,34 @@ class SQLLogicContext:
                 'allow_unredacted_secrets',
                 'duckdb_api',
             ]:
-                # Can not be set after initialization
+                # Cannot be set after initialization
                 continue
-            if name in ['profiling_mode', 'enable_profiling']:
-                # FIXME: 'profiling_mode' becomes "standard" when requested, but that's not actually the default setting
+
+            # If enable_profiling is NULL, skip setting custom_profiling_settings to not
+            # accidentally enable profiling.
+            # In that case, custom_profiling_settings is set to the default value anyway.
+            if name == "custom_profiling_settings" and "enable_profiling" not in old_settings:
                 continue
-            query = f"set {name}='{value}'"
+
+            query = f"SET {name}='{value}'"
             con.execute(query)
 
     def execute_set(self, statement: Set):
         option = statement.header.parameters[0]
-        string_set = (
-            self.runner.ignore_error_messages
-            if option == "ignore_error_messages"
-            else self.runner.always_fail_error_messages
-        )
-        string_set.clear()
-        string_set = statement.error_messages
+        if option == 'ignore_error_messages':
+            string_set = (
+                self.runner.ignore_error_messages
+                if option == "ignore_error_messages"
+                else self.runner.always_fail_error_messages
+            )
+            string_set.clear()
+            string_set = statement.error_messages
+        elif option == 'seed':
+            con = self.get_connection()
+            con.execute(f"SELECT SETSEED({statement.header.parameters[1]})")
+            self.runner.skip_reload = True
+        else:
+            self.skiptest(f"SET '{option}' is not implemented!")
 
     def execute_hash_threshold(self, statement: HashThreshold):
         self.runner.hash_threshold = statement.threshold
@@ -942,6 +1020,9 @@ class SQLLogicContext:
     def execute_statement(self, statement: Statement):
         assert isinstance(statement, Statement)
         conn = self.get_connection(statement.connection_name)
+        if not has_external_access(conn):
+            self.skiptest("enable_external_access is explicitly disabled by the test")
+
         sql_query = '\n'.join(statement.lines)
         sql_query = self.replace_keywords(sql_query)
 
@@ -959,13 +1040,19 @@ class SQLLogicContext:
             if expected_result.lines == None:
                 return
             expected = '\n'.join(expected_result.lines)
-            # Sanitize the expected error
-            if expected.startswith('Dependency Error: '):
-                expected = expected.split('Dependency Error: ')[1]
-            if expected not in str(e):
-                self.fail(
-                    f"Query failed, but did not produce the right error: {expected}\nInstead it produced: {str(e)}"
-                )
+            if is_regex(expected):
+                if not matches_regex(expected, str(e)):
+                    self.fail(
+                        f"Query failed, but did not produce the right error: {expected}\nInstead it produced: {str(e)}"
+                    )
+            else:
+                # Sanitize the expected error
+                if expected.startswith('Dependency Error: '):
+                    expected = expected.split('Dependency Error: ')[1]
+                if expected not in str(e):
+                    self.fail(
+                        f"Query failed, but did not produce the right error: {expected}\nInstead it produced: {str(e)}"
+                    )
 
     def check_require(self, statement: Require) -> RequireResult:
         not_an_extension = [
@@ -982,7 +1069,7 @@ class SQLLogicContext:
             "exact_vector_size",
             "block_size",
             "skip_reload",
-            "noalternativeverify",
+            "no_alternative_verify",
         ]
         param = statement.header.parameters[0].lower()
         if param in not_an_extension:
@@ -993,7 +1080,9 @@ class SQLLogicContext:
                 return RequireResult.PRESENT
             if param == 'exact_vector_size':
                 required_vector_size = int(statement.header.parameters[1])
-                return duckdb.__standard_vector_size__ == required_vector_size
+                if duckdb.__standard_vector_size__ == required_vector_size:
+                    return RequireResult.PRESENT
+                return RequireResult.MISSING
             if param == 'skip_reload':
                 self.runner.skip_reload = True
                 return RequireResult.PRESENT
@@ -1016,6 +1105,15 @@ class SQLLogicContext:
                 return RequireResult.MISSING
             return RequireResult.PRESENT
 
+        allow_unsigned_extensions = connection.execute(
+            "select value::BOOLEAN from duckdb_settings() where name == 'allow_unsigned_extensions'"
+        ).fetchone()[0]
+        if param == "allow_unsigned_extensions":
+            if allow_unsigned_extensions == False:
+                # If extension validation is turned on (that is allow_unsigned_extensions=False), skip test
+                return RequireResult.MISSING
+            return RequireResult.PRESENT
+
         excluded_from_autoloading = True
         for ext in self.runner.AUTOLOADABLE_EXTENSIONS:
             if ext == param:
@@ -1105,7 +1203,10 @@ class SQLLogicContext:
                     context.remove_keyword(key)
 
             loop_context = SQLLogicContext(self.pool, self.runner, statements, self.keywords.copy(), update_value)
-            loop_context.execute()
+            try:
+                loop_context.execute()
+            except TestException:
+                self.error = loop_context.error
         else:
             contexts: Dict[Tuple[str, int], Any] = {}
             for val in range(loop.start, loop.end):
@@ -1237,6 +1338,8 @@ class SQLLogicContext:
                     if self.runner.skip_active() and statement.__class__ != Unskip:
                         # Keep skipping until Unskip is found
                         continue
+                    if statement.get_decorators() != []:
+                        self.skiptest("Decorators are not supported yet")
                     method = self.STATEMENTS.get(statement.__class__)
                     if not method:
                         self.skiptest("Not supported by the runner")
