@@ -3,10 +3,9 @@
 #include "duckdb/common/printer.hpp"
 #include "duckdb/common/types/column/column_data_collection.hpp"
 #include "duckdb/common/vector_operations/vector_operations.hpp"
+#include "duckdb/original/std/sstream.hpp"
 #include "utf8proc_wrapper.hpp"
 
-#include <sstream>
-
 namespace duckdb {
 
 const idx_t BoxRenderer::SPLIT_COLUMN = idx_t(-1);
