@@ -2,17 +2,13 @@
 
 namespace duckdb {
 
-EncryptionState::EncryptionState() {
+EncryptionState::EncryptionState(const std::string *key) {
 	// abstract class, no implementation needed
 }
 
 EncryptionState::~EncryptionState() {
 }
 
-bool EncryptionState::IsOpenSSL() {
-	throw NotImplementedException("EncryptionState Abstract Class is called");
-}
-
 void EncryptionState::InitializeEncryption(duckdb::const_data_ptr_t iv, duckdb::idx_t iv_len, const std::string *key) {
 	throw NotImplementedException("EncryptionState Abstract Class is called");
 }
