@@ -21,6 +21,9 @@
 #include "duckdb/storage/metadata/metadata_reader.hpp"
 #include "duckdb/storage/table/scan_state.hpp"
 #include "duckdb/storage/table_io_manager.hpp"
+#include "duckdb/execution/index/art/art_scanner.hpp"
+#include "duckdb/execution/index/art/art_merger.hpp"
+#include "duckdb/execution/index/art/art_operator.hpp"
 
 namespace duckdb {
 
@@ -45,7 +48,7 @@ ART::ART(const string &name, const IndexConstraintType index_constraint_type, co
          const shared_ptr<array<unsafe_unique_ptr<FixedSizeAllocator>, ALLOCATOR_COUNT>> &allocators_ptr,
          const IndexStorageInfo &info)
     : BoundIndex(name, ART::TYPE_NAME, index_constraint_type, column_ids, table_io_manager, unbound_expressions, db),
-      allocators(allocators_ptr), owns_data(false) {
+      allocators(allocators_ptr), owns_data(false), verify_max_key_len(false) {
 
 	// FIXME: Use the new byte representation function to support nested types.
 	for (idx_t i = 0; i < types.size(); i++) {
@@ -70,6 +73,12 @@ ART::ART(const string &name, const IndexConstraintType index_constraint_type, co
 		}
 	}
 
+	if (types.size() > 1) {
+		verify_max_key_len = true;
+	} else if (types[0] == PhysicalType::VARCHAR) {
+		verify_max_key_len = true;
+	}
+
 	// Initialize the allocators.
 	SetPrefixCount(info);
 	if (!allocators) {
@@ -380,11 +389,25 @@ void GenerateKeysInternal(ArenaAllocator &allocator, DataChunk &input, unsafe_ve
 template <>
 void ART::GenerateKeys<>(ArenaAllocator &allocator, DataChunk &input, unsafe_vector<ARTKey> &keys) {
 	GenerateKeysInternal<false>(allocator, input, keys);
+	if (!verify_max_key_len) {
+		return;
+	}
+	auto max_len = MAX_KEY_LEN * idx_t(prefix_count);
+	for (idx_t i = 0; i < input.size(); i++) {
+		keys[i].VerifyKeyLength(max_len);
+	}
 }
 
 template <>
 void ART::GenerateKeys<true>(ArenaAllocator &allocator, DataChunk &input, unsafe_vector<ARTKey> &keys) {
 	GenerateKeysInternal<true>(allocator, input, keys);
+	if (!verify_max_key_len) {
+		return;
+	}
+	auto max_len = MAX_KEY_LEN * idx_t(prefix_count);
+	for (idx_t i = 0; i < input.size(); i++) {
+		keys[i].VerifyKeyLength(max_len);
+	}
 }
 
 void ART::GenerateKeyVectors(ArenaAllocator &allocator, DataChunk &input, Vector &row_ids, unsafe_vector<ARTKey> &keys,
@@ -431,7 +454,14 @@ bool ART::ConstructInternal(const unsafe_vector<ARTKey> &keys, const unsafe_vect
 		if (row_id_count == 1) {
 			Leaf::New(ref, row_ids[section.start].GetRowId());
 		} else {
-			Leaf::New(*this, ref, row_ids, section.start, row_id_count);
+			// Loop and insert the row IDs.
+			// We cannot use Construct in the leaf because row IDs are not sorted.
+			ArenaAllocator arena(BufferAllocator::Get(db));
+			for (idx_t i = section.start; i < section.start + row_id_count; i++) {
+				ARTOperator::Insert(arena, *this, ref, row_ids[i], 0, row_ids[i], GateStatus::GATE_SET, nullptr,
+				                    IndexAppendMode::DEFAULT);
+			}
+			ref.get().SetGateStatus(GateStatus::GATE_SET);
 		}
 		return true;
 	}
@@ -488,10 +518,10 @@ ErrorData ART::Insert(IndexLock &l, DataChunk &chunk, Vector &row_ids, IndexAppe
 	D_ASSERT(row_ids.GetType().InternalType() == ROW_TYPE);
 	auto row_count = chunk.size();
 
-	ArenaAllocator allocator(BufferAllocator::Get(db));
+	ArenaAllocator arena(BufferAllocator::Get(db));
 	unsafe_vector<ARTKey> keys(row_count);
 	unsafe_vector<ARTKey> row_id_keys(row_count);
-	GenerateKeyVectors(allocator, chunk, row_ids, keys, row_id_keys);
+	GenerateKeyVectors(arena, chunk, row_ids, keys, row_id_keys);
 
 	optional_ptr<ART> delete_art;
 	if (info.delete_index) {
@@ -507,7 +537,8 @@ ErrorData ART::Insert(IndexLock &l, DataChunk &chunk, Vector &row_ids, IndexAppe
 		if (keys[i].Empty()) {
 			continue;
 		}
-		conflict_type = Insert(tree, keys[i], 0, row_id_keys[i], tree.GetGateStatus(), delete_art, info.append_mode);
+		conflict_type = ARTOperator::Insert(arena, *this, tree, keys[i], 0, row_id_keys[i], GateStatus::GATE_NOT_SET,
+		                                    delete_art, info.append_mode);
 		if (conflict_type != ARTConflictType::NO_CONFLICT) {
 			conflict_idx = i;
 			break;
@@ -545,7 +576,7 @@ ErrorData ART::Insert(IndexLock &l, DataChunk &chunk, Vector &row_ids, IndexAppe
 		if (keys[i].Empty()) {
 			continue;
 		}
-		D_ASSERT(Lookup(tree, keys[i], 0));
+		D_ASSERT(ARTOperator::Lookup(*this, tree, keys[i], 0));
 	}
 #endif
 	return ErrorData();
@@ -581,148 +612,6 @@ void ART::VerifyAppend(DataChunk &chunk, IndexAppendInfo &info, optional_ptr<Con
 	VerifyConstraint(chunk, info, local_manager);
 }
 
-void ART::InsertIntoEmpty(Node &node, const ARTKey &key, const idx_t depth, const ARTKey &row_id,
-                          const GateStatus status) {
-	D_ASSERT(depth <= key.len);
-	D_ASSERT(!node.HasMetadata());
-
-	if (status == GateStatus::GATE_SET) {
-		Leaf::New(node, row_id.GetRowId());
-		return;
-	}
-
-	reference<Node> ref(node);
-	auto count = key.len - depth;
-
-	Prefix::New(*this, ref, key, depth, count);
-	Leaf::New(ref, row_id.GetRowId());
-}
-
-ARTConflictType ART::InsertIntoInlined(Node &node, const ARTKey &key, const idx_t depth, const ARTKey &row_id,
-                                       const GateStatus status, optional_ptr<ART> delete_art,
-                                       const IndexAppendMode append_mode) {
-
-	if (!IsUnique() || append_mode == IndexAppendMode::INSERT_DUPLICATES) {
-		Leaf::InsertIntoInlined(*this, node, row_id, depth, status);
-		return ARTConflictType::NO_CONFLICT;
-	}
-
-	if (!delete_art) {
-		if (append_mode == IndexAppendMode::IGNORE_DUPLICATES) {
-			return ARTConflictType::NO_CONFLICT;
-		}
-		return ARTConflictType::CONSTRAINT;
-	}
-
-	// Lookup in the delete_art.
-	auto delete_leaf = delete_art->Lookup(delete_art->tree, key, 0);
-	if (!delete_leaf) {
-		return ARTConflictType::CONSTRAINT;
-	}
-
-	// The row ID has changed.
-	// Thus, the local index has a newer (local) row ID, and this is a constraint violation.
-	D_ASSERT(delete_leaf->GetType() == NType::LEAF_INLINED);
-	auto deleted_row_id = delete_leaf->GetRowId();
-	auto this_row_id = node.GetRowId();
-	if (deleted_row_id != this_row_id) {
-		return ARTConflictType::CONSTRAINT;
-	}
-
-	// The deleted key and its row ID match the current key and its row ID.
-	Leaf::InsertIntoInlined(*this, node, row_id, depth, status);
-	return ARTConflictType::NO_CONFLICT;
-}
-
-ARTConflictType ART::InsertIntoNode(Node &node, const ARTKey &key, const idx_t depth, const ARTKey &row_id,
-                                    const GateStatus status, optional_ptr<ART> delete_art,
-                                    const IndexAppendMode append_mode) {
-	D_ASSERT(depth < key.len);
-	auto child = node.GetChildMutable(*this, key[depth]);
-
-	// Recurse, if a child exists at key[depth].
-	if (child) {
-		D_ASSERT(child->HasMetadata());
-		auto conflict_type = Insert(*child, key, depth + 1, row_id, status, delete_art, append_mode);
-		node.ReplaceChild(*this, key[depth], *child);
-		return conflict_type;
-	}
-
-	// Create an inlined prefix at key[depth].
-	if (status == GateStatus::GATE_SET) {
-		Node remainder;
-		auto byte = key[depth];
-		auto conflict_type = Insert(remainder, key, depth + 1, row_id, status, delete_art, append_mode);
-		Node::InsertChild(*this, node, byte, remainder);
-		return conflict_type;
-	}
-
-	// Insert an inlined leaf at key[depth].
-	Node leaf;
-	reference<Node> ref(leaf);
-
-	// Create the prefix.
-	if (depth + 1 < key.len) {
-		auto count = key.len - depth - 1;
-		Prefix::New(*this, ref, key, depth + 1, count);
-	}
-
-	// Create the inlined leaf.
-	Leaf::New(ref, row_id.GetRowId());
-	Node::InsertChild(*this, node, key[depth], leaf);
-	return ARTConflictType::NO_CONFLICT;
-}
-
-ARTConflictType ART::Insert(Node &node, const ARTKey &key, idx_t depth, const ARTKey &row_id, const GateStatus status,
-                            optional_ptr<ART> delete_art, const IndexAppendMode append_mode) {
-	if (!node.HasMetadata()) {
-		InsertIntoEmpty(node, key, depth, row_id, status);
-		return ARTConflictType::NO_CONFLICT;
-	}
-
-	// Enter a nested leaf.
-	if (status == GateStatus::GATE_NOT_SET && node.GetGateStatus() == GateStatus::GATE_SET) {
-		if (IsUnique()) {
-			// Unique indexes can have duplicates, if another transaction DELETE + INSERT
-			// the same key. In that case, the previous value must be kept alive until all
-			// other transactions do not depend on it anymore.
-
-			// We restrict this transactionality to two-value leaves, so any subsequent
-			// incoming transaction must fail here.
-			return ARTConflictType::TRANSACTION;
-		}
-		return Insert(node, row_id, 0, row_id, GateStatus::GATE_SET, delete_art, append_mode);
-	}
-
-	auto type = node.GetType();
-	switch (type) {
-	case NType::LEAF_INLINED: {
-		return InsertIntoInlined(node, key, depth, row_id, status, delete_art, append_mode);
-	}
-	case NType::LEAF: {
-		Leaf::TransformToNested(*this, node);
-		return Insert(node, key, depth, row_id, status, delete_art, append_mode);
-	}
-	case NType::NODE_7_LEAF:
-	case NType::NODE_15_LEAF:
-	case NType::NODE_256_LEAF: {
-		// Row IDs are unique, so there are never any duplicate byte conflicts here.
-		auto byte = key[Prefix::ROW_ID_COUNT];
-		Node::InsertChild(*this, node, byte);
-		return ARTConflictType::NO_CONFLICT;
-	}
-	case NType::NODE_4:
-	case NType::NODE_16:
-	case NType::NODE_48:
-	case NType::NODE_256:
-		return InsertIntoNode(node, key, depth, row_id, status, delete_art, append_mode);
-	case NType::PREFIX:
-		return Prefix::Insert(*this, node, key, depth, row_id, status, delete_art, append_mode);
-	default:
-		throw InternalException("Invalid node type for ART::Insert.");
-	}
-}
-
 //===--------------------------------------------------------------------===//
 // Drop and Delete
 //===--------------------------------------------------------------------===//
@@ -765,7 +654,7 @@ void ART::Delete(IndexLock &state, DataChunk &input, Vector &row_ids) {
 		if (keys[i].Empty()) {
 			continue;
 		}
-		auto leaf = Lookup(tree, keys[i], 0);
+		auto leaf = ARTOperator::Lookup(*this, tree, keys[i], 0);
 		if (leaf && leaf->GetType() == NType::LEAF_INLINED) {
 			D_ASSERT(leaf->GetRowId() != row_id_keys[i].GetRowId());
 		}
@@ -872,45 +761,8 @@ void ART::Erase(Node &node, reference<const ARTKey> key, idx_t depth, reference<
 // Point and range lookups
 //===--------------------------------------------------------------------===//
 
-const unsafe_optional_ptr<const Node> ART::Lookup(const Node &node, const ARTKey &key, idx_t depth) {
-	reference<const Node> ref(node);
-	while (ref.get().HasMetadata()) {
-
-		// Return the leaf.
-		if (ref.get().IsAnyLeaf() || ref.get().GetGateStatus() == GateStatus::GATE_SET) {
-			return unsafe_optional_ptr<const Node>(ref.get());
-		}
-
-		// Traverse the prefix.
-		if (ref.get().GetType() == NType::PREFIX) {
-			auto pos = Prefix::Traverse(*this, ref, key, depth);
-			if (pos.IsValid()) {
-				// Prefix mismatch, return nullptr.
-				return nullptr;
-			}
-			continue;
-		}
-
-		// Get the child node.
-		D_ASSERT(depth < key.len);
-		auto child = ref.get().GetChild(*this, key[depth]);
-
-		// No child at the matching byte, return nullptr.
-		if (!child) {
-			return nullptr;
-		}
-
-		// Continue in the child.
-		ref = *child;
-		D_ASSERT(ref.get().HasMetadata());
-		depth++;
-	}
-
-	return nullptr;
-}
-
 bool ART::SearchEqual(ARTKey &key, idx_t max_count, unsafe_vector<row_t> &row_ids) {
-	auto leaf = Lookup(tree, key, 0);
+	auto leaf = ARTOperator::Lookup(*this, tree, key, 0);
 	if (!leaf) {
 		return true;
 	}
@@ -976,6 +828,8 @@ bool ART::Scan(IndexScanState &state, const idx_t max_count, unsafe_vector<row_t
 	D_ASSERT(scan_state.values[0].type().InternalType() == types[0]);
 	ArenaAllocator arena_allocator(Allocator::Get(db));
 	auto key = ARTKey::CreateKey(arena_allocator, types[0], scan_state.values[0]);
+	auto max_len = MAX_KEY_LEN * prefix_count;
+	key.VerifyKeyLength(max_len);
 
 	if (scan_state.values[1].IsNull()) {
 		// Single predicate.
@@ -1000,6 +854,8 @@ bool ART::Scan(IndexScanState &state, const idx_t max_count, unsafe_vector<row_t
 	lock_guard<mutex> l(lock);
 	D_ASSERT(scan_state.values[1].type().InternalType() == types[0]);
 	auto upper_bound = ARTKey::CreateKey(arena_allocator, types[0], scan_state.values[1]);
+	upper_bound.VerifyKeyLength(max_len);
+
 	bool left_equal = scan_state.expressions[0] == ExpressionType ::COMPARE_GREATERTHANOREQUALTO;
 	bool right_equal = scan_state.expressions[1] == ExpressionType ::COMPARE_LESSTHANOREQUALTO;
 	return SearchCloseRange(key, upper_bound, left_equal, right_equal, max_count, row_ids);
@@ -1038,9 +894,11 @@ string ART::GenerateConstraintErrorMessage(VerifyExistenceType verify_type, cons
 	}
 	case VerifyExistenceType::DELETE_FK: {
 		// DELETE_FK that still exists in a FK table, i.e., not a valid delete.
-		return StringUtil::Format("Violates foreign key constraint because key \"%s\" is still referenced by a foreign "
-		                          "key in a different table",
-		                          key_name);
+		return StringUtil::Format(
+		    "Violates foreign key constraint because key \"%s\" is still referenced by a foreign "
+		    "key in a different table. If this is an unexpected constraint violation, please refer to our "
+		    "foreign key limitations in the documentation",
+		    key_name);
 	}
 	default:
 		throw NotImplementedException("Type not implemented for VerifyExistenceType");
@@ -1061,7 +919,7 @@ void ART::VerifyLeaf(const Node &leaf, const ARTKey &key, optional_ptr<ART> dele
 	// All leaves in the delete ART are inlined.
 	unsafe_optional_ptr<const Node> deleted_leaf;
 	if (delete_art) {
-		deleted_leaf = delete_art->Lookup(delete_art->tree, key, 0);
+		deleted_leaf = ARTOperator::Lookup(*delete_art, delete_art->tree, key, 0);
 	}
 
 	// The leaf is inlined, and there is no deleted leaf with the same key.
@@ -1091,16 +949,27 @@ void ART::VerifyLeaf(const Node &leaf, const ARTKey &key, optional_ptr<ART> dele
 		return;
 	}
 
+	// Fast path for FOREIGN KEY constraints.
+	// Up to here, the above code paths work implicitly for FKs, as the leaf is inlined.
 	// FIXME: proper foreign key + delete ART support.
-	// This implicitly works for foreign keys, as we do not have to consider the actual row IDs.
-	// We only need to know that there are conflicts (for now), as we still perform over-eager constraint checking.
+	if (index_constraint_type == IndexConstraintType::FOREIGN) {
+		D_ASSERT(!deleted_leaf);
+		// We don't handle FK conflicts in UPSERT, so the row ID should not matter.
+		if (manager.AddHit(i, MAX_ROW_ID)) {
+			conflict_idx = i;
+		}
+		return;
+	}
 
 	// Scan the two row IDs in the leaf.
 	Iterator it(*this);
 	it.FindMinimum(leaf);
 	ARTKey empty_key = ARTKey();
 	unsafe_vector<row_t> row_ids;
-	it.Scan(empty_key, 2, row_ids, false);
+	auto success = it.Scan(empty_key, 2, row_ids, false);
+	if (!success || row_ids.size() != 2) {
+		throw InternalException("VerifyLeaf expects exactly two row IDs to be scanned");
+	}
 
 	if (!deleted_leaf) {
 		if (manager.AddHit(i, row_ids[0]) || manager.AddHit(i, row_ids[1])) {
@@ -1148,7 +1017,7 @@ void ART::VerifyConstraint(DataChunk &chunk, IndexAppendInfo &info, ConflictMana
 			continue;
 		}
 
-		auto leaf = Lookup(tree, keys[i], 0);
+		auto leaf = ARTOperator::Lookup(*this, tree, keys[i], 0);
 		if (!leaf) {
 			if (manager.AddMiss(i)) {
 				conflict_idx = i;
@@ -1197,7 +1066,7 @@ void ART::TransformToDeprecated() {
 	if (deprecated_allocator) {
 		prefix_count = Prefix::DEPRECATED_COUNT;
 
-		D_ASSERT((*allocators)[idx]->IsEmpty());
+		D_ASSERT((*allocators)[idx]->Empty());
 		(*allocators)[idx]->Reset();
 		(*allocators)[idx] = std::move(deprecated_allocator);
 	}
@@ -1222,9 +1091,9 @@ IndexStorageInfo ART::GetStorageInfo(const case_insensitive_map_t<Value> &option
 
 #ifdef DEBUG
 	if (v1_0_0_storage) {
-		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_7_LEAF)]->IsEmpty());
-		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_15_LEAF)]->IsEmpty());
-		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_256_LEAF)]->IsEmpty());
+		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_7_LEAF)]->Empty());
+		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_15_LEAF)]->Empty());
+		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::NODE_256_LEAF)]->Empty());
 		D_ASSERT((*allocators)[Node::GetAllocatorIdx(NType::PREFIX)]->GetSegmentSize() ==
 		         Prefix::DEPRECATED_COUNT + Prefix::METADATA_SIZE);
 	}
@@ -1292,11 +1161,6 @@ void ART::SetPrefixCount(const IndexStorageInfo &info) {
 		return;
 	}
 
-	if (!IsUnique()) {
-		prefix_count = Prefix::ROW_ID_COUNT;
-		return;
-	}
-
 	idx_t compound_size = 0;
 	for (const auto &type : types) {
 		compound_size += GetTypeIdSize(type);
@@ -1359,7 +1223,52 @@ void ART::Vacuum(IndexLock &state) {
 	}
 
 	// Traverse the allocated memory of the tree to perform a vacuum.
-	tree.Vacuum(*this, indexes);
+	auto &art = *this;
+	auto handler = [&art, &indexes](Node &node) {
+		ARTHandlingResult result;
+		const auto type = node.GetType();
+		switch (type) {
+		case NType::LEAF_INLINED:
+			return ARTHandlingResult::SKIP;
+		case NType::LEAF: {
+			if (indexes.find(Node::GetAllocatorIdx(type)) == indexes.end()) {
+				return ARTHandlingResult::SKIP;
+			}
+			Leaf::DeprecatedVacuum(art, node);
+			return ARTHandlingResult::SKIP;
+		}
+		case NType::NODE_7_LEAF:
+		case NType::NODE_15_LEAF:
+		case NType::NODE_256_LEAF: {
+			result = ARTHandlingResult::SKIP;
+			break;
+		}
+		case NType::PREFIX:
+		case NType::NODE_4:
+		case NType::NODE_16:
+		case NType::NODE_48:
+		case NType::NODE_256: {
+			result = ARTHandlingResult::CONTINUE;
+			break;
+		}
+		default:
+			throw InternalException("invalid node type for Vacuum: %s", EnumUtil::ToString(type));
+		}
+
+		const auto idx = Node::GetAllocatorIdx(type);
+		auto &allocator = Node::GetAllocator(art, type);
+		const auto needs_vacuum = indexes.find(idx) != indexes.end() && allocator.NeedsVacuum(node);
+		if (needs_vacuum) {
+			const auto status = node.GetGateStatus();
+			node = allocator.VacuumPointer(node);
+			node.SetMetadata(static_cast<uint8_t>(type));
+			node.SetGateStatus(status);
+		}
+		return result;
+	};
+
+	ARTScanner<ARTScanHandling::EMPLACE, Node> scanner(*this, handler, tree);
+	scanner.Scan(handler);
 
 	// Finalize the vacuum operation.
 	FinalizeVacuum(indexes);
@@ -1369,13 +1278,33 @@ void ART::Vacuum(IndexLock &state) {
 // Merging
 //===--------------------------------------------------------------------===//
 
-void ART::InitializeMerge(unsafe_vector<idx_t> &upper_bounds) {
+void ART::InitializeMergeUpperBounds(unsafe_vector<idx_t> &upper_bounds) {
 	D_ASSERT(owns_data);
 	for (auto &allocator : *allocators) {
 		upper_bounds.emplace_back(allocator->GetUpperBoundBufferId());
 	}
 }
 
+void ART::InitializeMerge(Node &node, unsafe_vector<idx_t> &upper_bounds) {
+	D_ASSERT(node.HasMetadata());
+
+	auto handler = [&upper_bounds](Node &node) {
+		const auto type = node.GetType();
+		if (node.GetType() == NType::LEAF_INLINED) {
+			return ARTHandlingResult::CONTINUE;
+		}
+		if (type == NType::LEAF) {
+			throw InternalException("deprecated ART storage in InitializeMerge");
+		}
+		const auto idx = Node::GetAllocatorIdx(type);
+		node.IncreaseBufferId(upper_bounds[idx]);
+		return ARTHandlingResult::CONTINUE;
+	};
+
+	ARTScanner<ARTScanHandling::POP, Node> scanner(*this, handler, node);
+	scanner.Scan(handler);
+}
+
 bool ART::MergeIndexes(IndexLock &state, BoundIndex &other_index) {
 	auto &other_art = other_index.Cast<ART>();
 	if (!other_art.tree.HasMetadata()) {
@@ -1386,8 +1315,8 @@ bool ART::MergeIndexes(IndexLock &state, BoundIndex &other_index) {
 		if (tree.HasMetadata()) {
 			// Fully deserialize other_index, and traverse it to increment its buffer IDs.
 			unsafe_vector<idx_t> upper_bounds;
-			InitializeMerge(upper_bounds);
-			other_art.tree.InitMerge(other_art, upper_bounds);
+			InitializeMergeUpperBounds(upper_bounds);
+			other_art.InitializeMerge(other_art.tree, upper_bounds);
 		}
 
 		// Merge the node storage.
@@ -1398,9 +1327,15 @@ bool ART::MergeIndexes(IndexLock &state, BoundIndex &other_index) {
 
 	// Merge the ARTs.
 	D_ASSERT(tree.GetGateStatus() == other_art.tree.GetGateStatus());
-	if (!tree.Merge(*this, other_art.tree, tree.GetGateStatus())) {
-		return false;
+	if (tree.HasMetadata()) {
+		ArenaAllocator arena(Allocator::Get(db));
+		ARTMerger merger(arena, *this);
+		merger.Init(tree, other_art.tree);
+		return merger.Merge() == ARTConflictType::NO_CONFLICT;
 	}
+
+	tree = other_art.tree;
+	other_art.tree.Clear();
 	return true;
 }
 
@@ -1408,7 +1343,7 @@ bool ART::MergeIndexes(IndexLock &state, BoundIndex &other_index) {
 // Verification
 //===--------------------------------------------------------------------===//
 
-string ART::VerifyAndToString(IndexLock &state, const bool only_verify) {
+string ART::VerifyAndToString(IndexLock &l, const bool only_verify) {
 	return VerifyAndToStringInternal(only_verify);
 }
 
@@ -1419,7 +1354,7 @@ string ART::VerifyAndToStringInternal(const bool only_verify) {
 	return "[empty]";
 }
 
-void ART::VerifyAllocations(IndexLock &state) {
+void ART::VerifyAllocations(IndexLock &l) {
 	return VerifyAllocationsInternal();
 }
 
@@ -1441,6 +1376,12 @@ void ART::VerifyAllocationsInternal() {
 #endif
 }
 
+void ART::VerifyBuffers(IndexLock &l) {
+	for (auto &allocator : *allocators) {
+		allocator->VerifyBuffers();
+	}
+}
+
 constexpr const char *ART::TYPE_NAME;
 
 } // namespace duckdb
