@@ -173,13 +173,13 @@ void CSVSniffer::GenerateStateMachineSearchSpace(vector<unique_ptr<ColumnCountSc
 }
 
 // Returns true if a comment is acceptable
-bool AreCommentsAcceptable(const ColumnCountResult &result, idx_t num_cols, bool comment_set_by_user) {
-	if (comment_set_by_user) {
+bool AreCommentsAcceptable(const ColumnCountResult &result, idx_t num_cols, const CSVReaderOptions &options) {
+	if (options.dialect_options.state_machine_options.comment.IsSetByUser()) {
 		return true;
 	}
 	// For a comment to be acceptable, we want 3/5th's the majority of unmatched in the columns
 	constexpr double min_majority = 0.6;
-	// detected comments, are all lines that started with a comment character.
+	// detected comments are all lines that started with a comment character.
 	double detected_comments = 0;
 	// If at least one comment is a full line comment
 	bool has_full_line_comment = false;
@@ -192,7 +192,9 @@ bool AreCommentsAcceptable(const ColumnCountResult &result, idx_t num_cols, bool
 				has_full_line_comment = true;
 				valid_comments++;
 			}
-			if (result.column_counts[i].number_of_columns == num_cols && result.column_counts[i].is_mid_comment) {
+			if ((result.column_counts[i].number_of_columns == num_cols ||
+			     (result.column_counts[i].number_of_columns <= num_cols && options.null_padding)) &&
+			    result.column_counts[i].is_mid_comment) {
 				valid_comments++;
 			}
 		}
@@ -212,13 +214,13 @@ bool AreCommentsAcceptable(const ColumnCountResult &result, idx_t num_cols, bool
 	return valid_comments / detected_comments >= min_majority;
 }
 
-void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner, idx_t &rows_read,
-                                         idx_t &best_consistent_rows, idx_t &prev_padding_count,
-                                         idx_t &min_ignored_rows) {
+void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner, CandidateStats &stats,
+                                         vector<unique_ptr<ColumnCountScanner>> &successful_candidates) {
 	// The sniffed_column_counts variable keeps track of the number of columns found for each row
 	auto &sniffed_column_counts = scanner->ParseChunk();
 	idx_t dirty_notes = 0;
 	idx_t dirty_notes_minus_comments = 0;
+	idx_t empty_lines = 0;
 	if (sniffed_column_counts.error) {
 		if (!scanner->error_handler->HasError(MAXIMUM_LINE_SIZE)) {
 			all_fail_max_line_size = false;
@@ -232,7 +234,7 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 	idx_t consistent_rows = 0;
 	idx_t num_cols = sniffed_column_counts.result_position == 0 ? 1 : sniffed_column_counts[0].number_of_columns;
 	const bool ignore_errors = options.ignore_errors.GetValue();
-	// If we are ignoring errors and not null_padding , we pick the most frequent number of columns as the right one
+	// If we are ignoring errors and not null_padding, we pick the most frequent number of columns as the right one
 	const bool use_most_frequent_columns = ignore_errors && !options.null_padding;
 	if (use_most_frequent_columns) {
 		num_cols = sniffed_column_counts.GetMostFrequentColumnCount();
@@ -242,8 +244,8 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 	idx_t ignored_rows = 0;
 	const bool allow_padding = options.null_padding;
 	bool first_valid = false;
-	if (sniffed_column_counts.result_position > rows_read) {
-		rows_read = sniffed_column_counts.result_position;
+	if (sniffed_column_counts.result_position > stats.rows_read) {
+		stats.rows_read = sniffed_column_counts.result_position;
 	}
 	if (set_columns.IsCandidateUnacceptable(num_cols, options.null_padding, ignore_errors,
 	                                        sniffed_column_counts[0].last_value_always_empty)) {
@@ -279,9 +281,10 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 				sniffed_column_counts.state_machine.dialect_options.rows_until_header = row;
 			}
 			padding_count = 0;
-			// we use the maximum amount of num_cols that we find
+			// we use the maximum number of num_cols that we find
 			num_cols = sniffed_column_counts[row].number_of_columns;
-			dirty_notes = row;
+			dirty_notes = row + sniffed_column_counts[row].empty_lines;
+			empty_lines = sniffed_column_counts[row].empty_lines;
 			dirty_notes_minus_comments = dirty_notes - comment_rows;
 			header_idx = row;
 			consistent_rows = 1;
@@ -289,7 +292,8 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 			if (!first_valid) {
 				first_valid = true;
 				sniffed_column_counts.state_machine.dialect_options.rows_until_header = row;
-				dirty_notes = row;
+				dirty_notes = row + sniffed_column_counts[row].empty_lines;
+				empty_lines = sniffed_column_counts[row].empty_lines;
 				dirty_notes_minus_comments = dirty_notes - comment_rows;
 				num_cols = sniffed_column_counts[row].number_of_columns;
 			}
@@ -311,24 +315,26 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 	consistent_rows += padding_count;
 
 	// Whether there are more values (rows) available that are consistent, exceeding the current best.
-	const bool more_values = consistent_rows > best_consistent_rows && num_cols >= max_columns_found;
+	const bool more_values = consistent_rows > stats.best_consistent_rows && num_cols >= max_columns_found;
 
-	const bool more_columns = consistent_rows == best_consistent_rows && num_cols > max_columns_found;
+	const bool more_columns = consistent_rows == stats.best_consistent_rows && num_cols > max_columns_found;
 
 	// If additional padding is required when compared to the previous padding count.
-	const bool require_more_padding = padding_count > prev_padding_count;
+	const bool require_more_padding = padding_count > stats.prev_padding_count;
 
 	// If less padding is now required when compared to the previous padding count.
-	const bool require_less_padding = padding_count < prev_padding_count;
+	const bool require_less_padding = padding_count < stats.prev_padding_count;
 
 	// If there was only a single column before, and the new number of columns exceeds that.
-	const bool single_column_before = max_columns_found < 2 && num_cols > max_columns_found * candidates.size();
+	const bool single_column_before =
+	    max_columns_found < 2 && num_cols > max_columns_found * successful_candidates.size();
 
 	// If the number of rows is consistent with the calculated value after accounting for skipped rows and the
 	// start row.
-	const bool rows_consistent =
-	    consistent_rows + (dirty_notes_minus_comments - options.dialect_options.skip_rows.GetValue()) + comment_rows ==
-	    sniffed_column_counts.result_position - options.dialect_options.skip_rows.GetValue();
+	const bool rows_consistent = consistent_rows +
+	                                 (dirty_notes_minus_comments - options.dialect_options.skip_rows.GetValue()) +
+	                                 comment_rows - empty_lines ==
+	                             sniffed_column_counts.result_position - options.dialect_options.skip_rows.GetValue();
 	// If there are more than one consistent row.
 	const bool more_than_one_row = consistent_rows > 1;
 
@@ -336,14 +342,14 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 	const bool more_than_one_column = num_cols > 1;
 
 	// If the start position is valid.
-	const bool start_good = !candidates.empty() &&
-	                        dirty_notes <= candidates.front()->GetStateMachine().dialect_options.skip_rows.GetValue();
+	const bool start_good =
+	    !successful_candidates.empty() &&
+	    dirty_notes <= successful_candidates.front()->GetStateMachine().dialect_options.skip_rows.GetValue();
 
 	// If padding happened but it is not allowed.
 	const bool invalid_padding = !allow_padding && padding_count > 0;
 
-	const bool comments_are_acceptable = AreCommentsAcceptable(
-	    sniffed_column_counts, num_cols, options.dialect_options.state_machine_options.comment.IsSetByUser());
+	const bool comments_are_acceptable = AreCommentsAcceptable(sniffed_column_counts, num_cols, options);
 
 	const bool quoted =
 	    scanner->ever_quoted &&
@@ -360,44 +366,44 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 	// If rows are consistent and no invalid padding happens, this is the best suitable candidate if one of the
 	// following is valid:
 	// - There's a single column before.
-	// - There are more values and no additional padding is required.
+	// - There are more values, and no additional padding is required.
 	// - There's more than one column and less padding is required.
 	if (columns_match_set && (rows_consistent || (set_columns.IsSet() && ignore_errors)) &&
 	    (single_column_before || ((more_values || more_columns) && !require_more_padding) ||
-	     (more_than_one_column && require_less_padding) || quoted) &&
+	     (more_than_one_column && require_less_padding) || (quoted && comment_rows == 0)) &&
 	    !invalid_padding && comments_are_acceptable) {
-		if (!candidates.empty() && set_columns.IsSet() && max_columns_found == set_columns.Size() &&
-		    consistent_rows <= best_consistent_rows) {
+		if (!successful_candidates.empty() && set_columns.IsSet() && max_columns_found == set_columns.Size() &&
+		    consistent_rows <= stats.best_consistent_rows) {
 			// We have a candidate that fits our requirements better
-			if (candidates.front()->ever_quoted || !scanner->ever_quoted) {
+			if (successful_candidates.front()->ever_quoted || !scanner->ever_quoted) {
 				return;
 			}
 		}
 		auto &sniffing_state_machine = scanner->GetStateMachine();
 
-		if (!candidates.empty() && candidates.front()->ever_quoted) {
+		if (!successful_candidates.empty() && successful_candidates.front()->ever_quoted) {
 			// Give preference to quoted boys.
 			if (!scanner->ever_quoted) {
 				return;
 			} else {
 				// Give preference to one that got escaped
-				if (!scanner->ever_escaped && candidates.front()->ever_escaped &&
+				if (!scanner->ever_escaped && successful_candidates.front()->ever_escaped &&
 				    sniffing_state_machine.dialect_options.state_machine_options.strict_mode.GetValue()) {
 					return;
 				}
-				if (best_consistent_rows == consistent_rows && num_cols >= max_columns_found) {
+				if (stats.best_consistent_rows == consistent_rows && num_cols >= max_columns_found) {
 					// If both have not been escaped, this might get solved later on.
 					sniffing_state_machine.dialect_options.num_cols = num_cols;
-					candidates.emplace_back(std::move(scanner));
+					successful_candidates.emplace_back(std::move(scanner));
 					max_columns_found = num_cols;
 					return;
 				}
 			}
 		}
-		if (max_columns_found == num_cols && (ignored_rows > min_ignored_rows)) {
+		if (max_columns_found == num_cols && (ignored_rows > stats.min_ignored_rows)) {
 			return;
 		}
-		if (max_columns_found > 1 && num_cols > max_columns_found && consistent_rows < best_consistent_rows / 2 &&
+		if (max_columns_found > 1 && num_cols > max_columns_found && consistent_rows < stats.best_consistent_rows / 2 &&
 		    (options.null_padding || ignore_errors)) {
 			// When null_padding is true, we only give preference to a max number of columns if null padding is at least
 			// 50% as consistent as the best case scenario
@@ -406,39 +412,40 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 		if (quoted && num_cols < max_columns_found) {
 			if (scanner->ever_escaped &&
 			    sniffing_state_machine.dialect_options.state_machine_options.strict_mode.GetValue()) {
-				for (auto &candidate : candidates) {
+				for (auto &candidate : successful_candidates) {
 					if (candidate->ever_quoted && candidate->ever_escaped) {
 						return;
 					}
 				}
 
 			} else {
-				for (auto &candidate : candidates) {
+				for (auto &candidate : successful_candidates) {
 					if (candidate->ever_quoted) {
 						return;
 					}
 				}
 			}
 		}
-		best_consistent_rows = consistent_rows;
+		stats.best_consistent_rows = consistent_rows;
 		max_columns_found = num_cols;
-		prev_padding_count = padding_count;
-		min_ignored_rows = ignored_rows;
+		stats.prev_padding_count = padding_count;
+		stats.min_ignored_rows = ignored_rows;
 
 		if (options.dialect_options.skip_rows.IsSetByUser()) {
-			// If skip rows is set by user, and we found dirty notes, we only accept it if either null_padding or
+			// If skip rows are set by the user, and we found dirty notes, we only accept it if either null_padding or
 			// ignore_errors is set we have comments
-			if (dirty_notes != 0 && !options.null_padding && !options.ignore_errors.GetValue() && comment_rows == 0) {
+			if (dirty_notes - empty_lines != 0 && !options.null_padding && !options.ignore_errors.GetValue() &&
+			    comment_rows == 0) {
 				return;
 			}
 			sniffing_state_machine.dialect_options.skip_rows = options.dialect_options.skip_rows.GetValue();
 		} else if (!options.null_padding) {
 			sniffing_state_machine.dialect_options.skip_rows = dirty_notes_minus_comments;
 		}
-		candidates.clear();
+		successful_candidates.clear();
 		sniffing_state_machine.dialect_options.num_cols = num_cols;
 		lines_sniffed = sniffed_column_counts.result_position;
-		candidates.emplace_back(std::move(scanner));
+		successful_candidates.emplace_back(std::move(scanner));
 		return;
 	}
 	// If there's more than one row and column, the start is good, rows are consistent,
@@ -449,7 +456,7 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 		auto &sniffing_state_machine = scanner->GetStateMachine();
 
 		if (options.dialect_options.skip_rows.IsSetByUser()) {
-			// If skip rows is set by user, and we found dirty notes, we only accept it if either null_padding or
+			// If skip rows are set by the user, and we found dirty notes, we only accept it if either null_padding or
 			// ignore_errors is set
 			if (dirty_notes != 0 && !options.null_padding && !options.ignore_errors.GetValue()) {
 				return;
@@ -460,7 +467,7 @@ void CSVSniffer::AnalyzeDialectCandidate(unique_ptr<ColumnCountScanner> scanner,
 		}
 		sniffing_state_machine.dialect_options.num_cols = num_cols;
 		lines_sniffed = sniffed_column_counts.result_position;
-		candidates.emplace_back(std::move(scanner));
+		successful_candidates.emplace_back(std::move(scanner));
 	}
 }
 
@@ -481,8 +488,8 @@ bool CSVSniffer::RefineCandidateNextChunk(ColumnCountScanner &candidate) const {
 }
 
 void CSVSniffer::RefineCandidates() {
-	// It's very frequent that more than one dialect can parse a csv file, hence here we run one state machine
-	// fully on the whole sample dataset, when/if it fails we go to the next one.
+	// It's very frequent that more than one dialect can parse a csv file; hence here we run one state machine
+	// fully on the whole sample dataset, when/if it fails, we go to the next one.
 	if (candidates.empty()) {
 		// No candidates to refine
 		return;
@@ -587,22 +594,14 @@ NewLineIdentifier CSVSniffer::DetectNewLineDelimiter(CSVBufferManager &buffer_ma
 void CSVSniffer::DetectDialect() {
 	// Variables for Dialect Detection
 	DialectCandidates dialect_candidates(options.dialect_options.state_machine_options);
-	// Number of rows read
-	idx_t rows_read = 0;
-	// Best Number of consistent rows (i.e., presenting all columns)
-	idx_t best_consistent_rows = 0;
-	// If padding was necessary (i.e., rows are missing some columns, how many)
-	idx_t prev_padding_count = 0;
-	// Min number of ignores rows
-	idx_t best_ignored_rows = 0;
+	CandidateStats stats;
 	// Vector of CSV State Machines
 	vector<unique_ptr<ColumnCountScanner>> csv_state_machines;
 	// Step 1: Generate state machines
 	GenerateStateMachineSearchSpace(csv_state_machines, dialect_candidates);
 	// Step 2: Analyze all candidates on the first chunk
 	for (auto &state_machine : csv_state_machines) {
-		AnalyzeDialectCandidate(std::move(state_machine), rows_read, best_consistent_rows, prev_padding_count,
-		                        best_ignored_rows);
+		AnalyzeDialectCandidate(std::move(state_machine), stats, candidates);
 	}
 	// Step 3: Loop over candidates and find if they can still produce good results for the remaining chunks
 	RefineCandidates();
