@@ -1,11 +1,24 @@
+#include "duckdb/function/scalar/string_common.hpp"
 #include "duckdb/function/scalar/string_functions.hpp"
 
 #include "utf8proc.hpp"
 
 namespace duckdb {
 
-bool StripAccentsFun::IsAscii(const char *input, idx_t n) {
-	for (idx_t i = 0; i < n; i++) {
+bool IsAscii(const char *input, idx_t n) {
+	static constexpr uint64_t MASK = 0x8080808080808080U;
+
+	// Check 8 bytes at a time
+	idx_t i = 0;
+	for (; i + sizeof(uint64_t) <= n; i += sizeof(uint64_t)) {
+		if ((Load<uint64_t>(const_data_ptr_cast(input + i)) & MASK)) {
+			// non-ascii character in the next 8 bytes
+			return false;
+		}
+	}
+
+	// Less than 8 bytes remain
+	for (; i < n; i++) {
 		if (input[i] & 0x80) {
 			// non-ascii character
 			return false;
@@ -17,7 +30,7 @@ bool StripAccentsFun::IsAscii(const char *input, idx_t n) {
 struct StripAccentsOperator {
 	template <class INPUT_TYPE, class RESULT_TYPE>
 	static RESULT_TYPE Operation(INPUT_TYPE input, Vector &result) {
-		if (StripAccentsFun::IsAscii(input.GetData(), input.GetSize())) {
+		if (IsAscii(input.GetData(), input.GetSize())) {
 			return input;
 		}
 
@@ -41,8 +54,4 @@ ScalarFunction StripAccentsFun::GetFunction() {
 	return ScalarFunction("strip_accents", {LogicalType::VARCHAR}, LogicalType::VARCHAR, StripAccentsFunction);
 }
 
-void StripAccentsFun::RegisterFunction(BuiltinFunctions &set) {
-	set.AddFunction(StripAccentsFun::GetFunction());
-}
-
 } // namespace duckdb
