@@ -12,6 +12,7 @@
 #include "duckdb/parser/column_definition.hpp"
 #include "duckdb/parser/constraint.hpp"
 #include "duckdb/parser/parsed_data/parse_info.hpp"
+#include "duckdb/parser/result_modifier.hpp"
 
 namespace duckdb {
 
@@ -79,7 +80,12 @@ enum class AlterTableType : uint8_t {
 	SET_NOT_NULL = 8,
 	DROP_NOT_NULL = 9,
 	SET_COLUMN_COMMENT = 10,
-	ADD_CONSTRAINT = 11
+	ADD_CONSTRAINT = 11,
+	SET_PARTITIONED_BY = 12,
+	SET_SORTED_BY = 13,
+	ADD_FIELD = 14,
+	REMOVE_FIELD = 15,
+	RENAME_FIELD = 16
 };
 
 struct AlterTableInfo : public AlterInfo {
@@ -121,6 +127,29 @@ private:
 	RenameColumnInfo();
 };
 
+//===--------------------------------------------------------------------===//
+// RenameFieldInfo
+//===--------------------------------------------------------------------===//
+struct RenameFieldInfo : public AlterTableInfo {
+	RenameFieldInfo(AlterEntryData data, vector<string> column_path, string new_name_p);
+	~RenameFieldInfo() override;
+
+	//! Path to source field
+	vector<string> column_path;
+	//! Column new name
+	string new_name;
+
+public:
+	unique_ptr<AlterInfo> Copy() const override;
+	string ToString() const override;
+
+	void Serialize(Serializer &serializer) const override;
+	static unique_ptr<AlterTableInfo> Deserialize(Deserializer &deserializer);
+
+private:
+	RenameFieldInfo();
+};
+
 //===--------------------------------------------------------------------===//
 // RenameTableInfo
 //===--------------------------------------------------------------------===//
@@ -165,6 +194,31 @@ private:
 	explicit AddColumnInfo(ColumnDefinition new_column);
 };
 
+//===--------------------------------------------------------------------===//
+// AddFieldInfo
+//===--------------------------------------------------------------------===//
+struct AddFieldInfo : public AlterTableInfo {
+	AddFieldInfo(AlterEntryData data, vector<string> column_path, ColumnDefinition new_field, bool if_field_not_exists);
+	~AddFieldInfo() override;
+
+	//! The path to the struct
+	vector<string> column_path;
+	//! New field to add to the struct
+	ColumnDefinition new_field;
+	//! Whether or not an error should be thrown if the field exist
+	bool if_field_not_exists;
+
+public:
+	unique_ptr<AlterInfo> Copy() const override;
+	string ToString() const override;
+
+	void Serialize(Serializer &serializer) const override;
+	static unique_ptr<AlterTableInfo> Deserialize(Deserializer &deserializer);
+
+private:
+	explicit AddFieldInfo(ColumnDefinition new_column);
+};
+
 //===--------------------------------------------------------------------===//
 // RemoveColumnInfo
 //===--------------------------------------------------------------------===//
@@ -192,6 +246,29 @@ private:
 	RemoveColumnInfo();
 };
 
+//===--------------------------------------------------------------------===//
+// RemoveFieldInfo
+//===--------------------------------------------------------------------===//
+struct RemoveFieldInfo : public AlterTableInfo {
+	RemoveFieldInfo(AlterEntryData data, vector<string> column_path, bool if_column_exists, bool cascade);
+	~RemoveFieldInfo() override;
+
+	//! The path to the field to remove
+	vector<string> column_path;
+	//! Whether or not an error should be thrown if the column does not exist
+	bool if_column_exists;
+	//! Whether or not the column should be removed if a dependency conflict arises (used by GENERATED columns)
+	bool cascade;
+
+public:
+	unique_ptr<AlterInfo> Copy() const override;
+	string ToString() const override;
+	void Serialize(Serializer &serializer) const override;
+	static unique_ptr<AlterTableInfo> Deserialize(Deserializer &deserializer);
+
+private:
+	RemoveFieldInfo();
+};
 //===--------------------------------------------------------------------===//
 // ChangeColumnTypeInfo
 //===--------------------------------------------------------------------===//
@@ -367,4 +444,44 @@ private:
 	AddConstraintInfo();
 };
 
+//===--------------------------------------------------------------------===//
+// SetPartitionedByInfo
+//===--------------------------------------------------------------------===//
+struct SetPartitionedByInfo : public AlterTableInfo {
+	SetPartitionedByInfo(AlterEntryData data, vector<unique_ptr<ParsedExpression>> partition_keys);
+	~SetPartitionedByInfo() override;
+
+	//! The partition keys
+	vector<unique_ptr<ParsedExpression>> partition_keys;
+
+public:
+	unique_ptr<AlterInfo> Copy() const override;
+	string ToString() const override;
+	void Serialize(Serializer &serializer) const override;
+	static unique_ptr<AlterTableInfo> Deserialize(Deserializer &deserializer);
+
+private:
+	SetPartitionedByInfo();
+};
+
+//===--------------------------------------------------------------------===//
+// SetSortedByInfo
+//===--------------------------------------------------------------------===//
+struct SetSortedByInfo : public AlterTableInfo {
+	SetSortedByInfo(AlterEntryData data, vector<OrderByNode> orders);
+	~SetSortedByInfo() override;
+
+	//! The sort keys
+	vector<OrderByNode> orders;
+
+public:
+	unique_ptr<AlterInfo> Copy() const override;
+	string ToString() const override;
+	void Serialize(Serializer &serializer) const override;
+	static unique_ptr<AlterTableInfo> Deserialize(Deserializer &deserializer);
+
+private:
+	SetSortedByInfo();
+};
+
 } // namespace duckdb
