@@ -6,9 +6,10 @@
 
 namespace duckdb {
 
-BlockManager::BlockManager(BufferManager &buffer_manager, const optional_idx block_alloc_size_p)
+BlockManager::BlockManager(BufferManager &buffer_manager, const optional_idx block_alloc_size_p,
+                           const optional_idx block_header_size_p)
     : buffer_manager(buffer_manager), metadata_manager(make_uniq<MetadataManager>(*this, buffer_manager)),
-      block_alloc_size(block_alloc_size_p) {
+      block_alloc_size(block_alloc_size_p), block_header_size(block_header_size_p) {
 }
 
 shared_ptr<BlockHandle> BlockManager::RegisterBlock(block_id_t block_id) {
