@@ -269,14 +269,17 @@ ErrorData DuckTransactionManager::CommitTransaction(ClientContext &context, Tran
 	if (!error.HasError()) {
 		error = transaction.Commit(db, commit_id, std::move(commit_state));
 	}
+
 	if (error.HasError()) {
-		// commit unsuccessful: rollback the transaction instead
+		// COMMIT not successful: ROLLBACK.
 		checkpoint_decision = CheckpointDecision(error.Message());
 		transaction.commit_id = 0;
+
 		auto rollback_error = transaction.Rollback();
 		if (rollback_error.HasError()) {
-			throw FatalException("Failed to rollback transaction. Cannot continue operation.\nError: %s",
-			                     rollback_error.Message());
+			throw FatalException(
+			    "Failed to rollback transaction. Cannot continue operation.\nOriginal Error: %s\nRollback Error: %s",
+			    error.Message(), rollback_error.Message());
 		}
 	} else {
 		// check if catalog changes were made
