@@ -69,6 +69,10 @@ public:
 	virtual void ExecuteInternal(ExecuteContext &context) const = 0;
 	void Execute(ExecuteContext &context) const;
 
+	virtual bool SupportsConcurrent() const {
+		return false;
+	}
+
 private:
 	void RestartDatabase(ExecuteContext &context, Connection *&connection, string sql_query) const;
 };
@@ -82,6 +86,10 @@ public:
 
 public:
 	void ExecuteInternal(ExecuteContext &context) const override;
+
+	bool SupportsConcurrent() const override {
+		return true;
+	}
 };
 
 class Query : public Command {
@@ -96,6 +104,10 @@ public:
 
 public:
 	void ExecuteInternal(ExecuteContext &context) const override;
+
+	bool SupportsConcurrent() const override {
+		return true;
+	}
 };
 
 class RestartCommand : public Command {
@@ -124,6 +136,8 @@ public:
 	vector<duckdb::unique_ptr<Command>> loop_commands;
 
 	void ExecuteInternal(ExecuteContext &context) const override;
+
+	bool SupportsConcurrent() const override;
 };
 
 class ModeCommand : public Command {
@@ -167,4 +181,16 @@ private:
 	string extraction_path;
 };
 
+class LoadCommand : public Command {
+public:
+	LoadCommand(SQLLogicTestRunner &runner, string dbpath, bool readonly, const string &version = "");
+
+	string dbpath;
+	bool readonly;
+	string version;
+
+public:
+	void ExecuteInternal(ExecuteContext &context) const override;
+};
+
 } // namespace duckdb
