@@ -10,6 +10,7 @@
 #include "sqllogic_parser.hpp"
 #include "test_helpers.hpp"
 #include "sqllogic_test_logger.hpp"
+#include "duckdb/common/random_engine.hpp"
 
 #ifdef DUCKDB_OUT_OF_TREE
 #include DUCKDB_EXTENSION_HEADER
@@ -29,6 +30,10 @@ SQLLogicTestRunner::SQLLogicTestRunner(string dbpath) : dbpath(std::move(dbpath)
 		local_extension_repo = env_var;
 		config->options.autoload_known_extensions = true;
 	}
+	auto verify_vector = std::getenv("DUCKDB_DEBUG_VERIFY_VECTOR");
+	if (verify_vector) {
+		config->options.debug_verify_vector = EnumUtil::FromString<DebugVectorVerification>(verify_vector);
+	}
 }
 
 SQLLogicTestRunner::~SQLLogicTestRunner() {
@@ -49,7 +54,8 @@ SQLLogicTestRunner::~SQLLogicTestRunner() {
 
 void SQLLogicTestRunner::ExecuteCommand(duckdb::unique_ptr<Command> command) {
 	if (InLoop()) {
-		active_loops.back()->loop_commands.push_back(std::move(command));
+		auto &current_loop = *active_loops.back();
+		current_loop.loop_commands.push_back(std::move(command));
 	} else {
 		ExecuteContext context;
 		command->Execute(context);
@@ -60,7 +66,8 @@ void SQLLogicTestRunner::StartLoop(LoopDefinition definition) {
 	auto loop = make_uniq<LoopCommand>(*this, std::move(definition));
 	auto loop_ptr = loop.get();
 	if (InLoop()) {
-		active_loops.back()->loop_commands.push_back(std::move(loop));
+		auto &current_loop = *active_loops.back();
+		current_loop.loop_commands.push_back(std::move(loop));
 	} else {
 		// not in a loop yet: new top-level loop
 		top_level_loop = std::move(loop);
@@ -234,6 +241,7 @@ bool SQLLogicTestRunner::ForEachTokenReplace(const string &parameter, vector<str
 		result.push_back("bitpacking");
 		result.push_back("dictionary");
 		result.push_back("fsst");
+		result.push_back("dict_fsst");
 		result.push_back("alp");
 		result.push_back("alprd");
 		collection = true;
@@ -488,6 +496,17 @@ RequireResult SQLLogicTestRunner::CheckRequire(SQLLogicParser &parser, const vec
 #endif
 	}
 
+	if (param == "no_latest_storage") {
+#ifdef DUCKDB_LATEST_STORAGE
+		return RequireResult::MISSING;
+#elif defined(DUCKDB_ALTERNATIVE_VERIFY)
+		//! ALTERNATIVE_VERIFY also forces latest storage
+		return RequireResult::MISSING;
+#else
+		return RequireResult::PRESENT;
+#endif
+	}
+
 	if (param == "no_block_verification") {
 #ifdef DUCKDB_BLOCK_VERIFICATION
 		return RequireResult::MISSING;
@@ -497,13 +516,12 @@ RequireResult SQLLogicTestRunner::CheckRequire(SQLLogicParser &parser, const vec
 	}
 
 	if (param == "no_vector_verification") {
-#ifdef DUCKDB_VERIFY_VECTOR
-		return RequireResult::MISSING;
-#else
+		auto verify_vector = std::getenv("DUCKDB_DEBUG_VERIFY_VECTOR");
+		if (verify_vector) {
+			return RequireResult::MISSING;
+		}
 		return RequireResult::PRESENT;
-#endif
 	}
-
 	if (param == "no_extension_autoloading") {
 		if (params.size() < 2) {
 			parser.Fail("require no_extension_autoloading needs an explanation string");
@@ -522,6 +540,12 @@ RequireResult SQLLogicTestRunner::CheckRequire(SQLLogicParser &parser, const vec
 		}
 		return RequireResult::PRESENT;
 	}
+	if (param == "allow_unsigned_extensions") {
+		if (config->options.allow_unsigned_extensions) {
+			return RequireResult::PRESENT;
+		}
+		return RequireResult::MISSING;
+	}
 
 	bool excluded_from_autoloading = true;
 	for (const auto &ext : AUTOLOADABLE_EXTENSIONS) {
@@ -848,6 +872,19 @@ void SQLLogicTestRunner::ExecuteFile(string script) {
 					string_set->insert(current_string);
 					string_set->erase("");
 				}
+			} else if (token.parameters[0] == "seed") {
+				if (token.parameters.size() != 2) {
+					parser.Fail("set seed requires a single seed value");
+				}
+				Value seed(token.parameters[1]);
+				if (!seed.DefaultTryCastAs(LogicalType::DOUBLE)) {
+					parser.Fail("set seed requires a floating point parameter");
+				}
+				auto res = con->Query("SELECT SETSEED(" + seed.ToString() + ")");
+				if (res->HasError()) {
+					parser.Fail("Failed to set seed: %s", res->GetError());
+				}
+				skip_reload = true;
 			} else {
 				parser.Fail("unrecognized set parameter: %s", token.parameters[0]);
 			}
@@ -932,14 +969,33 @@ void SQLLogicTestRunner::ExecuteFile(string script) {
 			environment_variables[env_var] = env_actual;
 
 		} else if (token.type == SQLLogicTokenType::SQLLOGIC_LOAD) {
-			bool readonly = token.parameters.size() > 1 && token.parameters[1] == "readonly";
+			bool is_read_only = false;
+			if (token.parameters.size() > 1) {
+				auto param = token.parameters[1];
+				if (StringUtil::CIEquals("readonly", param)) {
+					is_read_only = true;
+				} else if (StringUtil::CIEquals("readwrite", param)) {
+					is_read_only = false;
+				} else {
+					parser.Fail(StringUtil::Format(
+					    "parameter to 'load' is invalid, received '%s', accepted options are 'readonly' and 'readwrite",
+					    param));
+				}
+			}
+
+			string version;
+			if (token.parameters.size() > 2) {
+				version = token.parameters[2];
+			}
+
 			string load_db_path;
 			if (!token.parameters.empty()) {
 				load_db_path = ReplaceKeywords(token.parameters[0]);
 			} else {
 				load_db_path = string();
 			}
-			auto command = make_uniq<LoadCommand>(*this, load_db_path, readonly);
+
+			auto command = make_uniq<LoadCommand>(*this, load_db_path, is_read_only, version);
 			ExecuteCommand(std::move(command));
 		} else if (token.type == SQLLogicTokenType::SQLLOGIC_RESTART) {
 			bool load_extensions = !(token.parameters.size() == 1 && token.parameters[0] == "no_extension_load");
