@@ -17,7 +17,7 @@ public:
 	explicit FileSystemObject(py::object fs, vector<string> filenames_p)
 	    : RegisteredObject(std::move(fs)), filenames(std::move(filenames_p)) {
 	}
-	virtual ~FileSystemObject() {
+	~FileSystemObject() override {
 		py::gil_scoped_acquire acquire;
 		// Assert that the 'obj' is a filesystem
 		D_ASSERT(py::isinstance(obj, DuckDBPyConnection::ImportCache()->duckdb.filesystem.ModifiedMemoryFileSystem()));
