@@ -11,18 +11,26 @@
 
 #include "duckdb_python/import_cache/python_import_cache_item.hpp"
 
+//! Note: This class is generated using scripts.
+//! If you need to add a new object to the cache you must:
+//! 1. adjust tools/pythonpkg/scripts/imports.py
+//! 2. run python3 tools/pythonpkg/scripts/generate_import_cache_json.py
+//! 3. run python3 tools/pythonpkg/scripts/generate_import_cache_cpp.py
+//! 4. run make format-main (the generator doesn't respect the formatting rules ;))
+
 namespace duckdb {
 
-struct NumpyCoreCacheItem : public PythonImportCacheItem {
+struct NumpyMaCacheItem : public PythonImportCacheItem {
 
 public:
-	NumpyCoreCacheItem(optional_ptr<PythonImportCacheItem> parent)
-	    : PythonImportCacheItem("core", parent), multiarray("multiarray", this) {
+	NumpyMaCacheItem(optional_ptr<PythonImportCacheItem> parent)
+	    : PythonImportCacheItem("ma", parent), masked("masked", this), masked_array("masked_array", this) {
 	}
-	~NumpyCoreCacheItem() override {
+	~NumpyMaCacheItem() override {
 	}
 
-	PythonImportCacheItem multiarray;
+	PythonImportCacheItem masked;
+	PythonImportCacheItem masked_array;
 };
 
 struct NumpyCacheItem : public PythonImportCacheItem {
@@ -32,7 +40,7 @@ public:
 
 public:
 	NumpyCacheItem()
-	    : PythonImportCacheItem("numpy"), core(this), ndarray("ndarray", this), datetime64("datetime64", this),
+	    : PythonImportCacheItem("numpy"), ma(this), ndarray("ndarray", this), datetime64("datetime64", this),
 	      generic("generic", this), int64("int64", this), bool_("bool_", this), byte("byte", this),
 	      ubyte("ubyte", this), short_("short", this), ushort_("ushort", this), intc("intc", this),
 	      uintc("uintc", this), int_("int_", this), uint("uint", this), longlong("longlong", this),
@@ -43,7 +51,7 @@ public:
 	~NumpyCacheItem() override {
 	}
 
-	NumpyCoreCacheItem core;
+	NumpyMaCacheItem ma;
 	PythonImportCacheItem ndarray;
 	PythonImportCacheItem datetime64;
 	PythonImportCacheItem generic;
@@ -66,6 +74,11 @@ public:
 	PythonImportCacheItem csingle;
 	PythonImportCacheItem cdouble;
 	PythonImportCacheItem clongdouble;
+
+protected:
+	bool IsRequired() const override final {
+		return false;
+	}
 };
 
 } // namespace duckdb
