@@ -1,3 +1,4 @@
+
 //===----------------------------------------------------------------------===//
 //                         DuckDB
 //
@@ -10,22 +11,26 @@
 
 #include "duckdb_python/import_cache/python_import_cache_item.hpp"
 
+//! Note: This class is generated using scripts.
+//! If you need to add a new object to the cache you must:
+//! 1. adjust tools/pythonpkg/scripts/imports.py
+//! 2. run python3 tools/pythonpkg/scripts/generate_import_cache_json.py
+//! 3. run python3 tools/pythonpkg/scripts/generate_import_cache_cpp.py
+//! 4. run make format-main (the generator doesn't respect the formatting rules ;))
+
 namespace duckdb {
 
 struct PolarsCacheItem : public PythonImportCacheItem {
+
 public:
 	static constexpr const char *Name = "polars";
 
 public:
+	PolarsCacheItem() : PythonImportCacheItem("polars"), DataFrame("DataFrame", this), LazyFrame("LazyFrame", this) {
+	}
 	~PolarsCacheItem() override {
 	}
-	virtual void LoadSubtypes(PythonImportCache &cache) override {
-		DataFrame.LoadAttribute("DataFrame", cache, *this);
-		LazyFrame.LoadAttribute("LazyFrame", cache, *this);
-	}
 
-public:
-	//! pandas.DataFrame
 	PythonImportCacheItem DataFrame;
 	PythonImportCacheItem LazyFrame;
 
