@@ -89,6 +89,11 @@ public:
 		}
 		return *connection;
 	}
+
+	bool ConnectionIsClosed() const {
+		return connection == nullptr;
+	}
+
 	const Connection &GetConnection() const {
 		if (!connection) {
 			ThrowConnectionException();
@@ -265,11 +270,14 @@ public:
 	unique_ptr<DuckDBPyRelation> FromParquet(const string &file_glob, bool binary_as_string, bool file_row_number,
 	                                         bool filename, bool hive_partitioning, bool union_by_name,
 	                                         const py::object &compression = py::none());
-
 	unique_ptr<DuckDBPyRelation> FromParquets(const vector<string> &file_globs, bool binary_as_string,
 	                                          bool file_row_number, bool filename, bool hive_partitioning,
 	                                          bool union_by_name, const py::object &compression = py::none());
 
+	unique_ptr<DuckDBPyRelation> FromParquetInternal(Value &&file_param, bool binary_as_string, bool file_row_number,
+	                                                 bool filename, bool hive_partitioning, bool union_by_name,
+	                                                 const py::object &compression = py::none());
+
 	unique_ptr<DuckDBPyRelation> FromArrow(py::object &arrow_object);
 
 	unordered_set<string> GetTableNames(const string &query);
@@ -288,6 +296,8 @@ public:
 
 	void Interrupt();
 
+	double QueryProgress();
+
 	ModifiedMemoryFileSystem &GetObjectFileSystem();
 
 	// cursor() is stupid
