@@ -23,10 +23,6 @@
 #endif
 #endif
 
-#define PyDateTime_TIMEDELTA_GET_DAYS(o)         (((PyDateTime_Delta *)(o))->days)
-#define PyDateTime_TIMEDELTA_GET_SECONDS(o)      (((PyDateTime_Delta *)(o))->seconds)
-#define PyDateTime_TIMEDELTA_GET_MICROSECONDS(o) (((PyDateTime_Delta *)(o))->microseconds)
-
 namespace duckdb {
 
 struct PyDictionary {
@@ -189,6 +185,7 @@ public:
 	int32_t day;
 
 public:
+	date_t ToDate();
 	Value ToDuckValue();
 };
 
@@ -198,7 +195,7 @@ public:
 
 public:
 	DUCKDB_API static int32_t GetUTCOffsetSeconds(py::handle &tzone_obj);
-	DUCKDB_API static interval_t GetUTCOffset(py::handle &tzone_obj);
+	DUCKDB_API static interval_t GetUTCOffset(py::handle &datetime, py::handle &tzone_obj);
 };
 
 struct PythonObject {
