@@ -10,7 +10,7 @@ unique_ptr<ProgressBarDisplay> JupyterProgressBarDisplay::Create() {
 
 void JupyterProgressBarDisplay::Initialize() {
 	auto &import_cache = *DuckDBPyConnection::ImportCache();
-	auto float_progress_attr = import_cache.ipywidgets().FloatProgress();
+	auto float_progress_attr = import_cache.ipywidgets.FloatProgress();
 	D_ASSERT(float_progress_attr.ptr() != nullptr);
 	// Initialize the progress bar
 	py::dict style;
@@ -20,7 +20,7 @@ void JupyterProgressBarDisplay::Initialize() {
 	progress_bar.attr("layout").attr("width") = "auto";
 
 	// Display the progress bar
-	auto display_attr = import_cache.IPython().display.display();
+	auto display_attr = import_cache.IPython.display.display();
 	D_ASSERT(display_attr.ptr() != nullptr);
 	display_attr(progress_bar);
 }
