@@ -6,8 +6,9 @@
 
 namespace duckdb {
 
-PythonFileHandle::PythonFileHandle(FileSystem &file_system, const string &path, const py::object &handle)
-    : FileHandle(file_system, path), handle(handle) {
+PythonFileHandle::PythonFileHandle(FileSystem &file_system, const string &path, const py::object &handle,
+                                   FileOpenFlags flags)
+    : FileHandle(file_system, path, flags), handle(handle) {
 }
 PythonFileHandle::~PythonFileHandle() {
 	try {
@@ -18,6 +19,24 @@ PythonFileHandle::~PythonFileHandle() {
 	}
 }
 
+const py::object &PythonFileHandle::GetHandle(const FileHandle &handle) {
+	return handle.Cast<PythonFileHandle>().handle;
+}
+
+void PythonFileHandle::Close() {
+	PythonGILWrapper gil;
+	handle.attr("close")();
+}
+
+PythonFilesystem::~PythonFilesystem() {
+	try {
+		PythonGILWrapper gil;
+		filesystem.dec_ref();
+		filesystem.release();
+	} catch (...) { // NOLINT
+	}
+}
+
 string PythonFilesystem::DecodeFlags(FileOpenFlags flags) {
 	// see https://stackoverflow.com/a/58925279 for truth table of python file modes
 	bool read = flags.OpenForReading();
@@ -66,7 +85,7 @@ unique_ptr<FileHandle> PythonFilesystem::OpenFile(const string &path, FileOpenFl
 	string flags_s = DecodeFlags(flags);
 
 	const auto &handle = filesystem.attr("open")(path, py::str(flags_s));
-	return make_uniq<PythonFileHandle>(*this, path, handle);
+	return make_uniq<PythonFileHandle>(*this, path, handle, flags);
 }
 
 int64_t PythonFilesystem::Write(FileHandle &handle, void *buffer, int64_t nr_bytes) {
@@ -109,7 +128,7 @@ bool PythonFilesystem::Exists(const string &filename, const char *func_name) con
 
 	return py::bool_(filesystem.attr(func_name)(filename));
 }
-vector<string> PythonFilesystem::Glob(const string &path, FileOpener *opener) {
+vector<OpenFileInfo> PythonFilesystem::Glob(const string &path, FileOpener *opener) {
 	PythonGILWrapper gil;
 
 	if (path.empty()) {
@@ -117,10 +136,11 @@ vector<string> PythonFilesystem::Glob(const string &path, FileOpener *opener) {
 	}
 	auto returner = py::list(filesystem.attr("glob")(path));
 
-	vector<string> results;
+	vector<OpenFileInfo> results;
 	auto unstrip_protocol = filesystem.attr("unstrip_protocol");
 	for (auto item : returner) {
-		results.push_back(py::str(unstrip_protocol(py::str(item))));
+		string file_path = py::str(unstrip_protocol(py::str(item)));
+		results.emplace_back(file_path);
 	}
 	return results;
 }
@@ -128,12 +148,14 @@ string PythonFilesystem::PathSeparator(const string &path) {
 	return "/";
 }
 int64_t PythonFilesystem::GetFileSize(FileHandle &handle) {
+	D_ASSERT(!py::gil_check());
 	// TODO: this value should be cached on the PythonFileHandle
 	PythonGILWrapper gil;
 
 	return py::int_(filesystem.attr("size")(handle.path));
 }
 void PythonFilesystem::Seek(duckdb::FileHandle &handle, uint64_t location) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	auto seek = PythonFileHandle::GetHandle(handle).attr("seek");
@@ -152,18 +174,21 @@ bool PythonFilesystem::CanHandleFile(const string &fpath) {
 	return false;
 }
 void PythonFilesystem::MoveFile(const string &source, const string &dest, optional_ptr<FileOpener> opener) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	auto move = filesystem.attr("mv");
 	move(py::str(source), py::str(dest));
 }
 void PythonFilesystem::RemoveFile(const string &filename, optional_ptr<FileOpener> opener) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	auto remove = filesystem.attr("rm");
 	remove(py::str(filename));
 }
 time_t PythonFilesystem::GetLastModifiedTime(FileHandle &handle) {
+	D_ASSERT(!py::gil_check());
 	// TODO: this value should be cached on the PythonFileHandle
 	PythonGILWrapper gil;
 
@@ -172,6 +197,7 @@ time_t PythonFilesystem::GetLastModifiedTime(FileHandle &handle) {
 	return py::int_(last_mod.attr("timestamp")());
 }
 void PythonFilesystem::FileSync(FileHandle &handle) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	PythonFileHandle::GetHandle(handle).attr("flush")();
@@ -180,11 +206,13 @@ bool PythonFilesystem::DirectoryExists(const string &directory, optional_ptr<Fil
 	return Exists(directory, "isdir");
 }
 void PythonFilesystem::RemoveDirectory(const string &directory, optional_ptr<FileOpener> opener) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	filesystem.attr("rm")(directory, py::arg("recursive") = true);
 }
 void PythonFilesystem::CreateDirectory(const string &directory, optional_ptr<FileOpener> opener) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	filesystem.attr("mkdir")(py::str(directory));
@@ -193,6 +221,7 @@ bool PythonFilesystem::ListFiles(const string &directory, const std::function<vo
                                  FileOpener *opener) {
 	static py::str DIRECTORY("directory");
 
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 	bool nonempty = false;
 
@@ -205,6 +234,7 @@ bool PythonFilesystem::ListFiles(const string &directory, const std::function<vo
 	return nonempty;
 }
 void PythonFilesystem::Truncate(FileHandle &handle, int64_t new_size) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	filesystem.attr("touch")(handle.path, py::arg("truncate") = true);
@@ -213,6 +243,7 @@ bool PythonFilesystem::IsPipe(const string &filename, optional_ptr<FileOpener> o
 	return false;
 }
 idx_t PythonFilesystem::SeekPosition(FileHandle &handle) {
+	D_ASSERT(!py::gil_check());
 	PythonGILWrapper gil;
 
 	return py::int_(PythonFileHandle::GetHandle(handle).attr("tell")());
