@@ -32,14 +32,20 @@ static void InitializeConsumers(py::class_<DuckDBPyRelation> &m) {
 
 	DefineMethod({"to_parquet", "write_parquet"}, m, &DuckDBPyRelation::ToParquet,
 	             "Write the relation object to a Parquet file in 'file_name'", py::arg("file_name"), py::kw_only(),
-	             py::arg("compression") = py::none());
+	             py::arg("compression") = py::none(), py::arg("field_ids") = py::none(),
+	             py::arg("row_group_size_bytes") = py::none(), py::arg("row_group_size") = py::none(),
+	             py::arg("overwrite") = py::none(), py::arg("per_thread_output") = py::none(),
+	             py::arg("use_tmp_file") = py::none(), py::arg("partition_by") = py::none(),
+	             py::arg("write_partition_columns") = py::none(), py::arg("append") = py::none());
 
 	DefineMethod(
 	    {"to_csv", "write_csv"}, m, &DuckDBPyRelation::ToCSV, "Write the relation object to a CSV file in 'file_name'",
 	    py::arg("file_name"), py::kw_only(), py::arg("sep") = py::none(), py::arg("na_rep") = py::none(),
 	    py::arg("header") = py::none(), py::arg("quotechar") = py::none(), py::arg("escapechar") = py::none(),
 	    py::arg("date_format") = py::none(), py::arg("timestamp_format") = py::none(), py::arg("quoting") = py::none(),
-	    py::arg("encoding") = py::none(), py::arg("compression") = py::none());
+	    py::arg("encoding") = py::none(), py::arg("compression") = py::none(), py::arg("overwrite") = py::none(),
+	    py::arg("per_thread_output") = py::none(), py::arg("use_tmp_file") = py::none(),
+	    py::arg("partition_by") = py::none(), py::arg("write_partition_columns") = py::none());
 
 	m.def("fetchone", &DuckDBPyRelation::FetchOne, "Execute and fetch a single row as a tuple")
 	    .def("fetchmany", &DuckDBPyRelation::FetchMany, "Execute and fetch the next set of rows as a list of tuples",
@@ -53,8 +59,8 @@ static void InitializeConsumers(py::class_<DuckDBPyRelation> &m) {
 	         py::arg("date_as_object") = false)
 	    .def("to_df", &DuckDBPyRelation::FetchDF, "Execute and fetch all rows as a pandas DataFrame", py::kw_only(),
 	         py::arg("date_as_object") = false)
-	    .def("fetch_df_chunk", &DuckDBPyRelation::FetchDFChunk, "Execute and fetch a chunk of the rows", py::kw_only(),
-	         py::arg("vectors_per_chunk"), py::arg("date_as_object"))
+	    .def("fetch_df_chunk", &DuckDBPyRelation::FetchDFChunk, "Execute and fetch a chunk of the rows",
+	         py::arg("vectors_per_chunk") = 1, py::kw_only(), py::arg("date_as_object") = false)
 	    .def("arrow", &DuckDBPyRelation::ToArrowTable, "Execute and fetch all rows as an Arrow Table",
 	         py::arg("batch_size") = 1000000)
 	    .def("fetch_arrow_table", &DuckDBPyRelation::ToArrowTable, "Execute and fetch all rows as an Arrow Table",
@@ -64,9 +70,16 @@ static void InitializeConsumers(py::class_<DuckDBPyRelation> &m) {
 	    .def("pl", &DuckDBPyRelation::ToPolars, "Execute and fetch all rows as a Polars DataFrame",
 	         py::arg("batch_size") = 1000000)
 	    .def("torch", &DuckDBPyRelation::FetchPyTorch, "Fetch a result as dict of PyTorch Tensors")
-	    .def("tf", &DuckDBPyRelation::FetchTF, "Fetch a result as dict of TensorFlow Tensors")
-	    .def("record_batch", &DuckDBPyRelation::ToRecordBatch,
-	         "Execute and return an Arrow Record Batch Reader that yields all rows", py::arg("batch_size") = 1000000)
+	    .def("tf", &DuckDBPyRelation::FetchTF, "Fetch a result as dict of TensorFlow Tensors");
+	const char *capsule_docs = R"(
+			Execute and return an ArrowArrayStream through the Arrow PyCapsule Interface.
+
+			https://arrow.apache.org/docs/dev/format/CDataInterface/PyCapsuleInterface.html
+		)";
+	m.def("__arrow_c_stream__", &DuckDBPyRelation::ToArrowCapsule, capsule_docs,
+	      py::arg("requested_schema") = py::none());
+	m.def("record_batch", &DuckDBPyRelation::ToRecordBatch,
+	      "Execute and return an Arrow Record Batch Reader that yields all rows", py::arg("batch_size") = 1000000)
 	    .def("fetch_arrow_reader", &DuckDBPyRelation::ToRecordBatch,
 	         "Execute and return an Arrow Record Batch Reader that yields all rows", py::arg("batch_size") = 1000000);
 }
@@ -139,7 +152,7 @@ static void InitializeAggregates(py::class_<DuckDBPyRelation> &m) {
 	         py::arg("projected_columns") = "")
 	    .def("sum", &DuckDBPyRelation::Sum, "Computes the sum of all values present in a given column",
 	         py::arg("column"), py::arg("groups") = "", py::arg("window_spec") = "", py::arg("projected_columns") = "")
-	    .def("unique", &DuckDBPyRelation::Unique, "Number of distinct values in a column.", py::arg("unique_aggr"));
+	    .def("unique", &DuckDBPyRelation::Unique, "Returns the distinct values in a column.", py::arg("unique_aggr"));
 	/* TODO: Approximate aggregate functions */
 	/* TODO: Statistical aggregate functions */
 	m.def("median", &DuckDBPyRelation::Median, "Computes the median over all values present in a given column",
@@ -207,7 +220,7 @@ static void InitializeSetOperators(py::class_<DuckDBPyRelation> &m) {
 
 static void InitializeMetaQueries(py::class_<DuckDBPyRelation> &m) {
 	m.def("describe", &DuckDBPyRelation::Describe,
-	      "Gives basic statistics (e.g., min,max) and if null exists for each column of the relation.")
+	      "Gives basic statistics (e.g., min, max) and if NULL exists for each column of the relation.")
 	    .def("explain", &DuckDBPyRelation::Explain, py::arg("type") = "standard");
 }
 
@@ -230,7 +243,8 @@ void DuckDBPyRelation::Initialize(py::handle &m) {
 	relation_module.def("filter", &DuckDBPyRelation::Filter, "Filter the relation object by the filter in filter_expr",
 	                    py::arg("filter_expr"));
 	DefineMethod({"select", "project"}, relation_module, &DuckDBPyRelation::Project,
-	             "Project the relation object by the projection in project_expr");
+	             "Project the relation object by the projection in project_expr", py::kw_only(),
+	             py::arg("groups") = "");
 	DefineMethod({"select_types", "select_dtypes"}, relation_module, &DuckDBPyRelation::ProjectFromTypes,
 	             "Select columns from the relation, by filtering based on type(s)", py::arg("types"));
 
@@ -250,13 +264,18 @@ void DuckDBPyRelation::Initialize(py::handle &m) {
 
 	    .def("join", &DuckDBPyRelation::Join,
 	         "Join the relation object with another relation object in other_rel using the join condition expression "
-	         "in join_condition. Types supported are 'inner' and 'left'",
+	         "in join_condition. Types supported are 'inner', 'left', 'right', 'outer', 'semi' and 'anti'",
 	         py::arg("other_rel"), py::arg("condition"), py::arg("how") = "inner")
+	    .def("cross", &DuckDBPyRelation::Cross, "Create cross/cartesian product of two relational objects",
+	         py::arg("other_rel"))
+
 	    .def("distinct", &DuckDBPyRelation::Distinct, "Retrieve distinct rows from this relation object")
 	    .def("limit", &DuckDBPyRelation::Limit,
 	         "Only retrieve the first n rows from this relation object, starting at offset", py::arg("n"),
 	         py::arg("offset") = 0)
 	    .def("insert", &DuckDBPyRelation::Insert, "Inserts the given values into the relation", py::arg("values"))
+	    .def("update", &DuckDBPyRelation::Update, "Update the given relation with the provided expressions",
+	         py::arg("set"), py::kw_only(), py::arg("condition") = py::none())
 
 	    // This should be deprecated in favor of a replacement scan
 	    .def("query", &DuckDBPyRelation::Query,
