@@ -56,7 +56,7 @@ def pytest_collection_modifyitems(config, items):
         if item.name in skipped_tests:
             # test is named specifically
             item.add_marker(skip_listed)
-        elif item.parent != None and item.parent.name in skipped_tests:
+        elif item.parent is not None and item.parent.name in skipped_tests:
             # the class is named specifically
             item.add_marker(skip_listed)
 
@@ -142,7 +142,7 @@ def convert_arrow_to_numpy_backend(df):
 def convert_to_numpy(df):
     if (
         pyarrow_dtypes_enabled
-        and pyarrow_dtype != None
+        and pyarrow_dtype is not None
         and any([True for x in df.dtypes if isinstance(x, pyarrow_dtype)])
     ):
         return convert_arrow_to_numpy_backend(df)
@@ -224,12 +224,15 @@ def require():
 # By making the scope 'function' we ensure that a new connection gets created for every function that uses the fixture
 @pytest.fixture(scope='function')
 def spark():
+    from spark_namespace import USE_ACTUAL_SPARK
+
     if not hasattr(spark, 'session'):
         # Cache the import
-        from duckdb.experimental.spark.sql import SparkSession as session
+        from spark_namespace.sql import SparkSession as session
 
         spark.session = session
-    return spark.session.builder.master(':memory:').appName('pyspark').getOrCreate()
+
+    return spark.session.builder.appName('pyspark').getOrCreate()
 
 
 @pytest.fixture(scope='function')
@@ -243,7 +246,22 @@ def duckdb_cursor():
 def integers(duckdb_cursor):
     cursor = duckdb_cursor
     cursor.execute('CREATE TABLE integers (i integer)')
-    cursor.execute('INSERT INTO integers VALUES (0),(1),(2),(3),(4),(5),(6),(7),(8),(9),(NULL)')
+    cursor.execute(
+        """
+        INSERT INTO integers VALUES
+            (0),
+            (1),
+            (2),
+            (3),
+            (4),
+            (5),
+            (6),
+            (7),
+            (8),
+            (9),
+            (NULL)
+    """
+    )
     yield
     cursor.execute("drop table integers")
 
