@@ -6,6 +6,12 @@ import pytest
 pa = pytest.importorskip("pyarrow")
 adbc_driver_manager = pytest.importorskip("adbc_driver_manager")
 
+if sys.version_info < (3, 9):
+    pytest.skip(
+        "Python Version must be higher or equal to 3.9 to run this test",
+        allow_module_level=True,
+    )
+
 try:
     adbc_driver_duckdb = pytest.importorskip("adbc_driver_duckdb.dbapi")
     con = adbc_driver_duckdb.connect()
