@@ -1,3 +1,4 @@
+import platform
 import threading
 import time
 
@@ -6,6 +7,10 @@ import pytest
 
 
 class TestConnectionInterrupt(object):
+    @pytest.mark.xfail(
+        condition=platform.system() == "Emscripten",
+        reason="threads not allowed on Emscripten",
+    )
     def test_connection_interrupt(self):
         conn = duckdb.connect()
 
@@ -17,7 +22,8 @@ class TestConnectionInterrupt(object):
         thread = threading.Thread(target=interrupt)
         thread.start()
         with pytest.raises(duckdb.InterruptException):
-            conn.execute("select count(*) from range(1000000000)").fetchall()
+            conn.execute("select count(*) from range(100000000000)").fetchall()
+        thread.join()
 
     def test_interrupt_closed_connection(self):
         conn = duckdb.connect()
