@@ -2,13 +2,23 @@ import duckdb
 import pytest
 from conftest import NumpyPandas, ArrowPandas
 
+pa = pytest.importorskip("pyarrow")
+
 
 def is_dunder_method(method_name: str) -> bool:
     if len(method_name) < 4:
         return False
+    if method_name.startswith('_pybind11'):
+        return True
     return method_name[:2] == '__' and method_name[:-3:-1] == '__'
 
 
+@pytest.fixture(scope="session")
+def tmp_database(tmp_path_factory):
+    database = tmp_path_factory.mktemp("databases", numbered=True) / "tmp.duckdb"
+    return database
+
+
 # This file contains tests for DuckDBPyConnection methods,
 # wrapped by the 'duckdb' module, to execute with the 'default_connection'
 class TestDuckDBConnection(object):
@@ -69,6 +79,21 @@ class TestDuckDBConnection(object):
             # 'tbl' no longer exists
             duckdb.table("tbl")
 
+    def test_cursor_lifetime(self):
+        con = duckdb.connect()
+
+        def use_cursors():
+            cursors = []
+            for _ in range(10):
+                cursors.append(con.cursor())
+
+            for cursor in cursors:
+                print("closing cursor")
+                cursor.close()
+
+        use_cursors()
+        con.close()
+
     def test_df(self):
         ref = [([1, 2, 3],)]
         duckdb.execute("select [1,2,3]")
@@ -130,7 +155,10 @@ class TestDuckDBConnection(object):
         assert duckdb.query(statements[1]).fetchall() == [(21,)]
         assert duckdb.execute(statements[1]).fetchall() == [(21,)]
 
-        with pytest.raises(duckdb.InvalidInputException, match='Prepared statement needs 1 parameters, 0 given'):
+        with pytest.raises(
+            duckdb.InvalidInputException,
+            match='Values were not provided for the following prepared statement parameters: 1',
+        ):
             duckdb.execute(statements[0])
         assert duckdb.execute(statements[0], {'1': 42}).fetchall() == [(42,)]
 
@@ -242,15 +270,6 @@ class TestDuckDBConnection(object):
     def test_from_query(self):
         assert None != duckdb.from_query
 
-    def test_from_substrait(self):
-        assert None != duckdb.from_substrait
-
-    def test_get_substrait(self):
-        assert None != duckdb.get_substrait
-
-    def test_get_substrait_json(self):
-        assert None != duckdb.get_substrait_json
-
     def test_get_table_names(self):
         assert None != duckdb.get_table_names
 
@@ -274,6 +293,25 @@ class TestDuckDBConnection(object):
         con.sql("create table tbl as select * from relation")
         assert con.table('tbl').fetchall() == [([5, 4, 3],)]
 
+    def test_unregister_problematic_behavior(self, duckdb_cursor):
+        # We have a VIEW called 'vw' in the Catalog
+        duckdb_cursor.execute("create temporary view vw as from range(100)")
+        assert duckdb_cursor.execute("select * from vw").fetchone() == (0,)
+
+        # Create a registered object called 'vw'
+        arrow_result = duckdb_cursor.execute("select 42").arrow()
+        with pytest.raises(duckdb.CatalogException, match='View with name "vw" already exists'):
+            duckdb_cursor.register('vw', arrow_result)
+
+        # Temporary views take precedence over registered objects
+        assert duckdb_cursor.execute("select * from vw").fetchone() == (0,)
+
+        # Decide that we're done with this registered object..
+        duckdb_cursor.unregister('vw')
+
+        # This should not have affected the existing view:
+        assert duckdb_cursor.execute("select * from vw").fetchone() == (0,)
+
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
     def test_relation_out_of_scope(self, pandas):
         def temporary_scope():
@@ -320,6 +358,14 @@ class TestDuckDBConnection(object):
     def test_interrupt(self):
         assert None != duckdb.interrupt
 
+    def test_wrap_shadowing(self):
+        pd = NumpyPandas()
+        import duckdb
+
+        df = pd.DataFrame({"a": [1, 2, 3]})
+        res = duckdb.sql("from df").fetchall()
+        assert res == [(1,), (2,), (3,)]
+
     def test_wrap_coverage(self):
         con = duckdb.default_connection
 
@@ -330,6 +376,18 @@ class TestDuckDBConnection(object):
             # Assert that every method of DuckDBPyConnection is wrapped by the 'duckdb' module
             assert method in dir(duckdb)
 
+    def test_connect_with_path(self, tmp_database):
+        import pathlib
+
+        assert isinstance(tmp_database, pathlib.Path)
+        con = duckdb.connect(tmp_database)
+        assert con.sql("select 42").fetchall() == [(42,)]
+
+        with pytest.raises(
+            duckdb.InvalidInputException, match="Please provide either a str or a pathlib.Path, not <class 'int'>"
+        ):
+            con = duckdb.connect(5)
+
     def test_set_pandas_analyze_sample_size(self):
         con = duckdb.connect(":memory:named", config={"pandas_analyze_sample": 0})
         res = con.sql("select current_setting('pandas_analyze_sample')").fetchone()
