@@ -322,17 +322,13 @@ class TestResolveObjectColumns(object):
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
     def test_map_duplicate(self, pandas, duckdb_cursor):
         x = pandas.DataFrame([[{'key': ['a', 'a', 'b'], 'value': [4, 0, 4]}]])
-        with pytest.raises(
-            duckdb.InvalidInputException, match="Dict->Map conversion failed because 'key' list contains duplicates"
-        ):
+        with pytest.raises(duckdb.InvalidInputException, match="Map keys must be unique."):
             duckdb_cursor.sql("select * from x").show()
 
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
     def test_map_nullkey(self, pandas, duckdb_cursor):
         x = pandas.DataFrame([[{'key': [None, 'a', 'b'], 'value': [4, 0, 4]}]])
-        with pytest.raises(
-            duckdb.InvalidInputException, match="Dict->Map conversion failed because 'key' list contains None"
-        ):
+        with pytest.raises(duckdb.InvalidInputException, match="Map keys can not be NULL."):
             converted_col = duckdb_cursor.sql("select * from x").df()
 
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
@@ -345,9 +341,7 @@ class TestResolveObjectColumns(object):
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
     def test_map_fallback_nullkey(self, pandas, duckdb_cursor):
         x = pandas.DataFrame([[{'a': 4, None: 0, 'c': 4}], [{'a': 4, None: 0, 'd': 4}]])
-        with pytest.raises(
-            duckdb.InvalidInputException, match="Dict->Map conversion failed because 'key' list contains None"
-        ):
+        with pytest.raises(duckdb.InvalidInputException, match="Map keys can not be NULL."):
             converted_col = duckdb_cursor.sql("select * from x").df()
 
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
@@ -358,9 +352,7 @@ class TestResolveObjectColumns(object):
                 [{'key': None, None: 5}],
             ]
         )
-        with pytest.raises(
-            duckdb.InvalidInputException, match="Dict->Map conversion failed because 'key' list contains None"
-        ):
+        with pytest.raises(duckdb.InvalidInputException, match="Map keys can not be NULL."):
             converted_col = duckdb_cursor.sql("select * from x").df()
 
     @pytest.mark.parametrize('pandas', [NumpyPandas(), ArrowPandas()])
