@@ -26,6 +26,7 @@ from duckdb.typing import (
     TIMESTAMP_MS,
     TIMESTAMP_NS,
     TIMESTAMP_S,
+    DuckDBPyType,
     TIME,
     TIME_TZ,
     TIMESTAMP_TZ,
@@ -43,7 +44,7 @@ class TestType(object):
         # todo: add tests with invalid type_str
 
     def test_primitive_types(self):
-        assert str(SQLNULL) == 'NULL'
+        assert str(SQLNULL) == '"NULL"'
         assert str(BOOLEAN) == 'BOOLEAN'
         assert str(TINYINT) == 'TINYINT'
         assert str(UTINYINT) == 'UTINYINT'
@@ -87,6 +88,12 @@ class TestType(object):
         type = duckdb.struct_type([BIGINT, BOOLEAN])
         assert str(type) == 'STRUCT(v1 BIGINT, v2 BOOLEAN)'
 
+    def test_incomplete_struct_type(self):
+        with pytest.raises(
+            duckdb.InvalidInputException, match='Could not convert empty dictionary to a duckdb STRUCT type'
+        ):
+            type = duckdb.typing.DuckDBPyType(dict())
+
     def test_map_type(self):
         type = duckdb.map_type(duckdb.sqltype("BIGINT"), duckdb.sqltype("DECIMAL(10, 2)"))
         assert str(type) == 'MAP(BIGINT, DECIMAL(10,2))'
@@ -118,9 +125,9 @@ class TestType(object):
         type = duckdb.list_type(list[str])
         assert str(type.child) == "VARCHAR[]"
 
-        mapping = {'VARCHAR': str, 'BIGINT': int, 'BLOB': bytes, 'BLOB': bytearray, 'BOOLEAN': bool, 'DOUBLE': float}
-        for expected, type in mapping.items():
-            res = duckdb.list_type(type)
+        mapping = {str: 'VARCHAR', int: 'BIGINT', bytes: 'BLOB', bytearray: 'BLOB', bool: 'BOOLEAN', float: 'DOUBLE'}
+        for duckdb_type, expected in mapping.items():
+            res = duckdb.list_type(duckdb_type)
             assert str(res.child) == expected
 
         res = duckdb.list_type({'a': str, 'b': int})
@@ -203,6 +210,10 @@ class TestType(object):
         res = duckdb.execute("select typeof($1)", [val]).fetchone()
         assert res == ('JSON',)
 
+    def test_struct_from_dict(self):
+        res = duckdb.list_type({'a': VARCHAR, 'b': VARCHAR})
+        assert res == 'STRUCT(a VARCHAR, b VARCHAR)[]'
+
     # NOTE: we can support this, but I don't think going through hoops for an outdated version of python is worth it
     @pytest.mark.skipif(sys.version_info < (3, 9), reason="python3.7 does not store Optional[..] in a recognized way")
     def test_optional(self):
@@ -227,3 +238,9 @@ class TestType(object):
     def test_optional_310(self):
         type = duckdb.typing.DuckDBPyType(str | None)
         assert type == 'VARCHAR'
+
+    def test_children_attribute(self):
+        assert DuckDBPyType('INTEGER[]').children == [('child', DuckDBPyType('INTEGER'))]
+        assert DuckDBPyType('INTEGER[2]').children == [('child', DuckDBPyType('INTEGER')), ('size', 2)]
+        assert DuckDBPyType('INTEGER[2][3]').children == [('child', DuckDBPyType('INTEGER[2]')), ('size', 3)]
+        assert DuckDBPyType("ENUM('a', 'b', 'c')").children == [('values', ['a', 'b', 'c'])]
