@@ -8,6 +8,8 @@
 
 #include "src/common/cgroups.cpp"
 
+#include "src/common/complex_json.cpp"
+
 #include "src/common/compressed_file_system.cpp"
 
 #include "src/common/constants.cpp"
@@ -34,16 +36,10 @@
 
 #include "src/common/hive_partitioning.cpp"
 
-#include "src/common/http_util.cpp"
-
 #include "src/common/pipe_file_system.cpp"
 
 #include "src/common/local_file_system.cpp"
 
-#include "src/common/multi_file_list.cpp"
-
-#include "src/common/multi_file_reader.cpp"
-
 #include "src/common/error_data.cpp"
 
 #include "src/common/opener_file_system.cpp"
